// ==============================================================
//                 ORBITER MODULE: Soyuz
//
// r7blockd.cpp
// ==============================================================

#define ORBITER_MODULE
#include "soyuz.h"
#include "math.h"

// ==============================================================
// Specialised vessel class r7blockd
// ==============================================================

// Constructor
r7blockd::r7blockd (OBJHANDLE hObj)
: VESSEL2(hObj)
{
	// preload mesh
	hblockdmesh = oapiLoadMeshGlobal ("r7blockd");
}

// ==============================================================
// Callback functions
// ==============================================================

// Set Tank class specs
void r7blockd::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetEnableFocus (false);
	// Tank cannot receive input focus

	SetSize (15.0);
	SetEmptyMass (BLOCKD_MASS);

	SetMaxFuelMass (0);
	// Note that the Tank instance is only created after separation from
	// the orbiter, so the actual fuel mass will always be much smaller

	SetISP (5000.0);

	SetMaxThrust (ENGINE_MAIN, 0);
	SetMaxThrust (ENGINE_RETRO, 0);
	SetMaxThrust (ENGINE_HOVER, 0);
	SetMaxThrust (ENGINE_ATTITUDE, 0);
	// Tank has no engines of its own

	SetCW (0.2, 0.3, 1.2, 1.2);
	SetCrossSections (_V(34.2, 32.2, 6.3));
	VECTOR3 rd = {0.7,0.7,0.3};
	SetRotDrag (rd);
	SetPMI (_V(28.8, 28.8, 1.5));
	SetPitchMomentScale (1e-4);
	SetBankMomentScale (1e-4);

	VECTOR3 co = {0,0,0};
	SetCameraOffset (co);
	// Note that the camera offset should not be required
	// since the Tank doesn't define a 'cockpit'

	SetCOG_elev (0.0);
	SetTouchdownPoints (_V(-2.70, -2.70, -0.9), _V(2.70, 0.0, -0.9), _V(2.70, -2.70, -0.9));
	SetLiftCoeffFunc (0);

	AddMesh (hblockdmesh);
}

// Simulation time step
void r7blockd::clbkPostStep (double simt, double simdt, double mjd)
{
	if (GetAltitude() < 5.0) oapiDeleteVessel (GetHandle());
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new r7blockd (hvessel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (r7blockd*)vessel;
}

// Vessel status initialisation
DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}
