// ==============================================================
//                 ORBITER MODULE: Soyuz
//
// soyufair2.cpp
// ==============================================================

#define ORBITER_MODULE
#include "soyuz.h"
#include "math.h"

// ==============================================================
// Specialised vessel class soyufair2
// ==============================================================

// Constructor
soyufair2::soyufair2 (OBJHANDLE hObj)
: VESSEL2(hObj)
{
	// preload mesh
	hsoyufair2mesh = oapiLoadMeshGlobal ("soyufair2");
}

// ==============================================================
// Callback functions
// ==============================================================

// Set class specs
void soyufair2::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetEnableFocus (false);
	// Tank cannot receive input focus

	SetSize (7.0);
	SetEmptyMass (FAIR2_MASS);

	SetMaxFuelMass (0);

	SetISP (5000.0);

	SetMaxThrust (ENGINE_MAIN, 0);
	SetMaxThrust (ENGINE_RETRO, 0);
	SetMaxThrust (ENGINE_HOVER, 0);
	SetMaxThrust (ENGINE_ATTITUDE, 0);
	// Tank has no engines of its own

	SetCW (0.2, 0.3, 1.2, 1.2);
	SetCrossSections (_V(26.1, 14.5, 4.0));
	VECTOR3 rd = {0.75,0.75,0.25};
	SetRotDrag (rd);
	SetPMI (_V(6.75, 6.0, 1.5));
	SetPitchMomentScale (1e-4);
	SetBankMomentScale (1e-4);

	VECTOR3 co = {0,0,0};
	SetCameraOffset (co);
	// Note that the camera offset should not be required
	// since this vessel doesn't define a 'cockpit'

	SetCOG_elev (0.0);
	SetTouchdownPoints (_V(-2.70, -2.70, -0.9), _V(2.70, 0.0, -0.9), _V(2.70, -2.70, -0.9));
	SetLiftCoeffFunc (0);

	AddMesh (hsoyufair2mesh);
}

// Simulation time step
void soyufair2::clbkPostStep (double simt, double simdt, double mjd)
{
	if (GetAltitude() < 5.0) oapiDeleteVessel (GetHandle());
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new soyufair2 (hvessel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (soyufair2*)vessel;
}

// Vessel status initialisation
DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}
