// ==============================================================
// $Id: soyuz.cpp,v 1.7 2004/09/07 00:35:29 dh219 Exp $
// Proton-K by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
	This is modified for MS VC++ 8.0
	It was originally done in CVEL
	I tried to keep the code intact enough to operate as originally designed.
	However, close only counts in horeshoes and handgrenades.
	Walter Keplinger - jpiscator1@yahoo.com

*/
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <stdio.h>
#include <fstream>
#include "soyuz.h"


struct {
	HINSTANCE hDLL;
} g_Param;

Soyuz::Soyuz (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)
{

	hbooster[0] = oapiLoadMeshGlobal ("r7blockb");
	hbooster[1] = oapiLoadMeshGlobal ("r7blockw");
	hbooster[2] = oapiLoadMeshGlobal ("r7blockg");
	hbooster[3] = oapiLoadMeshGlobal ("r7blockd");
	hstage1 = oapiLoadMeshGlobal ("r7blocka");
	hstage2 = oapiLoadMeshGlobal ("r7blocki");
	hfairtop = oapiLoadMeshGlobal ("SoyAShrd1");
	hfairbtm = oapiLoadMeshGlobal ("SoyAShrd2");
	hfairb = oapiLoadMeshGlobal ("soyufair1");
	hfairf = oapiLoadMeshGlobal ("soyufair2");
	hfairl = oapiLoadMeshGlobal ("soyshrd1");
	hfairr = oapiLoadMeshGlobal ("soyshrd2");
	histg[0] = oapiLoadMeshGlobal ("r7skirt3p1");
	histg[1] = oapiLoadMeshGlobal ("r7skirt3p2");
	histg[2] = oapiLoadMeshGlobal ("r7skirt3p3");
	hring = oapiLoadMeshGlobal( "soyuring" );
	hescape = oapiLoadMeshGlobal ("r7tmaer");
	htma = oapiLoadMesh("soyuz");
	hsoyservice = oapiLoadMesh("SoyASM");
	hsoyhs = oapiLoadMesh("ACRMHS");

	render_cockpit    = false;

	tex = oapiRegisterExhaustTexture ("Exhaust2");

	manual_separate=false;
	fair_attached = false;
	manned_launch = false;

	comshift = 4.0;

	stage = 2;


	ph_booster[0]	= NULL;	
	ph_booster[1]	= NULL;	
	ph_booster[2]	= NULL;	
	ph_booster[3]	= NULL;	
	ph_first        = NULL;
	ph_second       = NULL;

	abort_status     = DOOR_CLOSED;
	abort_proc       = 0.0;

	mesh_booster1 = MESH_UNDEFINED;
	mesh_booster2 = MESH_UNDEFINED;
	mesh_booster3 = MESH_UNDEFINED;
	mesh_booster4 = MESH_UNDEFINED;
	mesh_stage1   = MESH_UNDEFINED;
	mesh_stage2   = MESH_UNDEFINED;
	mesh_fairtop  = MESH_UNDEFINED;
	mesh_fairbtm  = MESH_UNDEFINED;
	mesh_fair1    = MESH_UNDEFINED;
	mesh_fair2    = MESH_UNDEFINED;
	mesh_fairl    = MESH_UNDEFINED;
	mesh_fairr    = MESH_UNDEFINED;
	mesh_fairf    = MESH_UNDEFINED;
	mesh_fairb    = MESH_UNDEFINED;
	mesh_istg1    = MESH_UNDEFINED;
	mesh_istg2    = MESH_UNDEFINED;
	mesh_istg3    = MESH_UNDEFINED;
	mesh_escape   = MESH_UNDEFINED;
	mesh_ring     = MESH_UNDEFINED;
	mesh_tma      = MESH_UNDEFINED;
	mesh_soyservice = MESH_UNDEFINED;
	mesh_soyhs    = MESH_UNDEFINED;

	OFF_BOOSTER[0] = _V(3.2,0,0);
	OFF_BOOSTER[1] = _V(0,3.2,0);
	OFF_BOOSTER[2] = _V(-3.2,0,0);
	OFF_BOOSTER[3] = _V(0,-3.2,0);

	OFF_STG1 = _V(0, 0, 4);
	OFF_STG2= _V(0,0,22.2);
	OFF_ISTG[0] = _V(0.665,0.33,19.19);
	OFF_ISTG[1] = _V(-0.665,0.33,19.19);
	OFF_ISTG[2] = _V(0,-1.05,19.19);
	OFF_LHF = _V(0,0,0); // these get assigned in refresh()
	OFF_RHF = _V(0,0,0);
	OFF_THF = _V(0,0,0);
	OFF_BHF = _V(0,0,0);
	OFF_SERVICE = _V(0,0,27.7);
	OFF_TMA = _V(0,0,29);

	OFF_ESCAPE = _V(0,0,38.5);
	OFF_ESC = _V(0,0,7.48);
	OFF_FLIGHT_ESCAPE = _V(0,0,16.3);

	OFF_RING= _V(0,0,27);

	OFF_ZERO = _V(0,0,0);
	OFF_FLIGHT_TMA = _V(0,0,6.8);
	OFF_FLIGHT_ISTG1 = _V(0.665,0.33,-3.01);
	OFF_FLIGHT_ISTG2 = _V(-0.665,0.33,-3.01);
	OFF_FLIGHT_ISTG3 = _V(0,-1.05,-3.01);
	OFF_FLIGHT_RING = _V(0,-1.05,4.8);
	OFF_FLIGHT_ORBIT = _V(0,0,2.42);
	OFF_FLIGHT_SERVICE = _V(0,0,5.5);

}

Soyuz::~Soyuz()
{
	delete th_first;
}

void Soyuz::SetAbortStage ()
{
	switch( stage ) {
	case( 0 ):
		ClearMeshes();	
		AddBooster1Visual(OFF_BOOSTER[0]);
		AddBooster2Visual(OFF_BOOSTER[1]);
		AddBooster3Visual(OFF_BOOSTER[2]);
		AddBooster4Visual(OFF_BOOSTER[3]);
		AddStage1Visual(OFF_STG1);
		AddStage2Visual(OFF_STG2);
		AddIstg1Visual(OFF_ISTG[0]);
		AddIstg2Visual(OFF_ISTG[1]);
		AddIstg3Visual(OFF_ISTG[2]);
		AddSoyServiceVisual(OFF_SERVICE);
		if( fair_attached ) {
			OFF_BHF = _V(0,  0, 27.665);
			AddFairingbtmVisual(OFF_BHF);
		}
		break;
	case( 1 ):
		ClearMeshes();	
		AddStage1Visual(OFF_STG1);
		AddStage2Visual(OFF_STG2);
		AddIstg1Visual(OFF_ISTG[0]);
		AddIstg2Visual(OFF_ISTG[1]);
		AddIstg3Visual(OFF_ISTG[2]);
		AddSoyServiceVisual(OFF_SERVICE);
		if( fair_attached ) {
			OFF_BHF = _V(0,  0, 27.665);
			AddFairingbtmVisual(OFF_BHF);
		}
		break;
	case( 2 ):
		ClearMeshes();	
		AddStage2Visual(OFF_FLIGHT_STG2);
		AddIstg1Visual(OFF_FLIGHT_ISTG1);
		AddIstg2Visual(OFF_FLIGHT_ISTG2);
		AddIstg3Visual(OFF_FLIGHT_ISTG3);
		AddSoyServiceVisual(OFF_FLIGHT_SERVICE);
		if( fair_attached ) {
			OFF_BHF = _V(0,  0, 5.465);
			AddFairingbtmVisual(OFF_BHF);
		}
		break;
	case( 3 ):
		AddStage2Visual(OFF_FLIGHT_STG2);
		AddTMAVisual(OFF_FLIGHT_TMA);
	}
}

void Soyuz::SetBoostStage ()
{

	double lev;
	lev = GetThrusterGroupLevel( THGROUP_MAIN );

	stage = 0;

	SetSize (50);
	SetEmptyMass ( MASS_STG0 + MASS_STG1 + MASS_STG2 + MASS_TMA );
	
//	char temp[50];
//	sprintf( temp, "0: %2.2f 1: %2.2f 2: %2.2f T: %2.2f", MASS_STG0, MASS_STG1, MASS_STG2, MASS_STG0 + MASS_STG1 + MASS_STG2 + CalculateMass() );
//	MessageBox( NULL,"","Debug",MB_ICONINFORMATION);
	
	ShiftCG (_V(0,0,1.0*comshift));
//	comshift = 0.0;
//	ShiftCG (_V(0,0,comshift));


	SetTouchdownPoints (_V(0,-1.0,-14), _V(-.7,.7,-14), _V(.7,.7,-14));
	SetPMI (_V(75,75,10));
	SetCrossSections (_V(105,105,25));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	ClearMeshes();	
	AddBooster1Visual(OFF_BOOSTER[0]);
	AddBooster2Visual(OFF_BOOSTER[1]);
	AddBooster3Visual(OFF_BOOSTER[2]);
	AddBooster4Visual(OFF_BOOSTER[3]);
	AddStage1Visual(OFF_STG1);
	AddStage2Visual(OFF_STG2);
	AddIstg1Visual(OFF_ISTG[0]);
	AddIstg2Visual(OFF_ISTG[1]);
	AddIstg3Visual(OFF_ISTG[2]);
	AddTMAVisual(OFF_TMA);

	if( !manned_launch ) {
		AddRingVisual(OFF_RING);
	}
	
	if( fair_attached ) {
		if( manned_launch ) {
			AddFairingtopVisual(OFF_THF);
			AddFairingbtmVisual(OFF_BHF);
			AddEscapeVisual(OFF_ESCAPE);
		} else {
			AddFairing1Visual(OFF_LHF);
			AddFairing2Visual(OFF_RHF);
		}
	}

//	ShiftCG(_V(0,0,-1.0*comshift));

	AddThrusters();

	SetDefaultPropellantResource (ph_first); 

	int i;
	for( i = 0 ; i < 4 ; i++ )
		th_main[i] = th_booster[i];
	th_main[4] = th_first;

	CreateThrusterGroup (th_main, 5, THGROUP_MAIN);

	SetThrusterGroupLevel( THGROUP_MAIN, lev );

}

void Soyuz::SetMainStage ()
{

	double lev;
	lev = GetThrusterGroupLevel( THGROUP_MAIN );

	stage = 1;

	SetSize (50);
	SetEmptyMass ( MASS_STG1 + MASS_STG2 + MASS_TMA);
	//sprintf( oapiDebugString(), "%2.2f", CalculateMass());
	
//	ShiftCG (_V(0,0,-1.0*comshift));
//	comshift = 8.0;
//	ShiftCG(_V(0,0,comshift));


	SetTouchdownPoints (_V(0,-2,10),_V(2,-2,-10),_V(-2,-2,-10));
	SetPMI (_V(75,75,5));
	SetCrossSections (_V(105,105,20));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	ClearMeshes();	
	
	AddStage1Visual(OFF_STG1);
	AddStage2Visual(OFF_STG2);
	AddIstg1Visual(OFF_ISTG[0]);
	AddIstg2Visual(OFF_ISTG[1]);
	AddIstg3Visual(OFF_ISTG[2]);
	AddTMAVisual(OFF_TMA);

	if( !manned_launch ) {
		AddRingVisual(OFF_RING);
	}
	
	if( fair_attached ) {
		if( manned_launch ) {
			AddFairingtopVisual(OFF_THF);
			AddFairingbtmVisual(OFF_BHF);
			AddEscapeVisual(OFF_ESCAPE);
		} else {
			AddFairing1Visual(OFF_LHF);
			AddFairing2Visual(OFF_RHF);
		}
	}

//	ShiftCG(_V(0,0,0-comshift));

	AddThrusters();

	int i;
	for( i = 0 ; i < 4 ; i++ ) {
		if (ph_booster[i]!=NULL) 
			DelPropellantResource (ph_booster[i]);
	}

	SetDefaultPropellantResource (ph_first); 

	th_main[0] = th_first;

	CreateThrusterGroup (th_main, 1, THGROUP_MAIN);

	SetThrusterGroupLevel( THGROUP_MAIN, lev );
}

void Soyuz::SetSecondStage() {

	double lev;
	lev = GetThrusterGroupLevel( THGROUP_MAIN );

	stage = 2;

	SetSize (20);
	SetEmptyMass ( MASS_STG2 + MASS_TMA);
	//sprintf( oapiDebugString(), "%2.2f", CalculateMass());
	
//	ShiftCG (_V(0,0,comshift));
	comshift = 22;
//	ShiftCG (_V(0,0,comshift));


	SetTouchdownPoints (_V(0,-2,10),_V(2,-2,-10),_V(-2,-2,-10));

	SetPMI (_V(23.75,23.75,2.25));
	SetCrossSections (_V(7.5,7.5,5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	ClearMeshes();	
	
	AddStage2Visual(OFF_ZERO);
	AddIstg1Visual(OFF_FLIGHT_ISTG1);
	AddIstg2Visual(OFF_FLIGHT_ISTG2);
	AddIstg3Visual(OFF_FLIGHT_ISTG3);
	AddTMAVisual(OFF_FLIGHT_TMA);

	if( !manned_launch ) {
		AddRingVisual(OFF_FLIGHT_RING);
	}
	
	if( fair_attached ) {
		if( manned_launch ) {
			AddFairingtopVisual(OFF_THF);
			AddFairingbtmVisual(OFF_BHF);
			AddEscapeVisual(OFF_ESCAPE);
		} else {
			AddFairing1Visual(OFF_LHF);
			AddFairing2Visual(OFF_RHF);
		}
	}
//	ShiftCG(_V(0,0,comshift));

	AddThrusters();

	if (ph_first!=NULL) 
		DelPropellantResource (ph_first	);

	SetDefaultPropellantResource (ph_second); 

	th_main[0] = th_second;
	th_main[1] = th_vern[0];
	th_main[2] = th_vern[1];
	th_main[3] = th_vern[2];
	th_main[4] = th_vern[3];

	CreateThrusterGroup (th_main, 5, THGROUP_MAIN);

	SetThrusterGroupLevel( THGROUP_MAIN, lev );


}

void Soyuz::SetPayloadStage ()
{
	double lev;
	lev = GetThrusterGroupLevel( THGROUP_MAIN );

	stage = 3;

	SetSize (20);
	SetEmptyMass ( MASS_STG2 );
	//sprintf( oapiDebugString(), "%2.2f", CalculateMass());
	
//	ShiftCG (_V(0,0,-1.0*comshift));
	comshift = 22.0;
//	ShiftCG (_V(0,0,comshift));


	SetTouchdownPoints (_V(0,-2,10),_V(2,-2,-10),_V(-2,-2,-10));

	SetPMI (_V(23.75,23.75,2.25));
	SetCrossSections (_V(7.5,7.5,5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	SetLiftCoeffFunc (0); 

	ClearMeshes();	
	
	AddStage2Visual(OFF_ZERO);
	AddIstg1Visual(OFF_FLIGHT_ISTG1);
	AddIstg2Visual(OFF_FLIGHT_ISTG2);
	AddIstg3Visual(OFF_FLIGHT_ISTG3);

	if( !manned_launch ) {
		AddRingVisual(OFF_FLIGHT_RING);
	}
	
//	ShiftCG(_V(0,0,comshift));

	AddThrusters();

	if (ph_first!=NULL) 
		DelPropellantResource (ph_first	);

	SetDefaultPropellantResource (ph_second); 

	th_main[0] = th_second;
	th_main[1] = th_vern[0];
	th_main[2] = th_vern[1];
	th_main[3] = th_vern[2];
	th_main[4] = th_vern[3];

	CreateThrusterGroup (th_main, 5, THGROUP_MAIN);

	SetThrusterGroupLevel( THGROUP_MAIN, lev );



}

void Soyuz::AbortLaunch ()
{
	if( manned_launch ) {
		if (stage < 2) {
			if (fair_attached)
				AddSoyServiceVisual(OFF_SERVICE);
			DelMesh (mesh_tma);
//			oapiDeleteMesh (htma);
		}
		if (stage > 1) {
			DelMesh (mesh_tma);
//			oapiDeleteMesh (htma);
		}
		if (ph_first !=NULL)
			SetPropellantMass(ph_first, 0.0);
		if (ph_second !=NULL)
			SetPropellantMass(ph_second, 0.0);
		if (ph_booster[0] !=NULL)
			SetPropellantMass(ph_booster[0], 0.0);
		if (ph_booster[1] !=NULL)
			SetPropellantMass(ph_booster[1], 0.0);
		if (ph_booster[2] !=NULL)
			SetPropellantMass(ph_booster[2], 0.0);
		if (ph_booster[3] !=NULL)
			SetPropellantMass(ph_booster[3], 0.0);
		if( fair_attached ) {
			DelMesh (mesh_fairtop);
			DelMesh (mesh_escape);
//			oapiDeleteMesh (hfairtop);
//			oapiDeleteMesh (hescape);
		} else {
			if (stage > 2) {
				DelMesh (mesh_tma);
//				oapiDeleteMesh (htma);
			}
		}
	}
	stage = stage + 10;
}

void Soyuz::AddBooster1Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_booster1 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_booster1 = AddMesh (hbooster[0], &ofs);
		mesh_booster1 = InsertMesh (hbooster[0], 1, &ofs);
		SetMeshVisibilityMode (mesh_booster1, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_booster1;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddBooster2Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_booster2 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_booster2 = AddMesh (hbooster[1], &ofs);
		mesh_booster2 = InsertMesh (hbooster[1], 2, &ofs);
		SetMeshVisibilityMode (mesh_booster2, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_booster2;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddBooster3Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_booster3 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_booster3 = AddMesh (hbooster[2], &ofs);
		mesh_booster3 = InsertMesh (hbooster[2], 3, &ofs);
		SetMeshVisibilityMode (mesh_booster3, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_booster3;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddBooster4Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_booster4 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_booster4 = AddMesh (hbooster[3], &ofs);
		mesh_booster4 = InsertMesh (hbooster[3], 4, &ofs);
		SetMeshVisibilityMode (mesh_booster4, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_booster4;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddStage1Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_stage1 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_stage1 = AddMesh (hstage1, &ofs);
		mesh_stage1 = InsertMesh (hstage1, 5, &ofs);
		SetMeshVisibilityMode (mesh_stage1, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_stage1;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddStage2Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_stage2 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_stage2 = AddMesh (hstage2, &ofs);
		mesh_stage2 = InsertMesh (hstage2, 6, &ofs);
		SetMeshVisibilityMode (mesh_stage2, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_stage2;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairing1Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fair1 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fair1 = AddMesh (hfair1, &ofs);
		mesh_fair1 = InsertMesh (hfair1, 7, &ofs);
		SetMeshVisibilityMode (mesh_fair1, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fair1;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairingtopVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairtop == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairtop = AddMesh (hfairtop, &ofs);
		mesh_fairtop = InsertMesh (hfairtop, 0, &ofs);
		SetMeshVisibilityMode (mesh_fairtop, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairtop;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairingbtmVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairbtm == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairbtm = AddMesh (hfairbtm, &ofs);
		mesh_fairbtm = InsertMesh (hfairbtm, 8, &ofs);
		SetMeshVisibilityMode (mesh_fairbtm, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairbtm;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairing2Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fair2 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fair2 = AddMesh (hfair2, &ofs);
		mesh_fair2 = InsertMesh (hfair2, 9, &ofs);
		SetMeshVisibilityMode (mesh_fair2, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fair2;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairinglVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairl == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairl = AddMesh (hfairl, &ofs);
		mesh_fairl = InsertMesh (hfairl, 10, &ofs);
		SetMeshVisibilityMode (mesh_fairl, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairl;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairingrVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairr == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairr = AddMesh (hfairr, &ofs);
		mesh_fairr = InsertMesh (hfairr, 11, &ofs);
		SetMeshVisibilityMode (mesh_fairr, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairr;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairingfVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairf == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairf = AddMesh (hfairf, &ofs);
		mesh_fairf = InsertMesh (hfairf, 12, &ofs);
		SetMeshVisibilityMode (mesh_fairf, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairf;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddFairingbVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_fairb == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_fairb = AddMesh (hfairb, &ofs);
		mesh_fairb = InsertMesh (hfairb, 13, &ofs);
		SetMeshVisibilityMode (mesh_fairb, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_fairb;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddIstg1Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_istg1 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_istg1 = AddMesh (histg[0], &ofs);
		mesh_istg1 = InsertMesh (histg[0], 14, &ofs);
		SetMeshVisibilityMode (mesh_istg1, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_istg1;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddIstg2Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_istg2 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_istg2 = AddMesh (histg[1], &ofs);
		mesh_istg2 = InsertMesh (histg[1], 15, &ofs);
		SetMeshVisibilityMode (mesh_istg2, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_istg1;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddIstg3Visual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_istg3 == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_istg3 = AddMesh (histg[2], &ofs);
		mesh_istg3 = InsertMesh (histg[2], 16, &ofs);
		SetMeshVisibilityMode (mesh_istg3, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_istg3;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddEscapeVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_escape == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_escape = AddMesh (hescape, &ofs);
		mesh_escape = InsertMesh (hescape, 17, &ofs);
		SetMeshVisibilityMode (mesh_escape, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_escape;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddRingVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_ring == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_ring = AddMesh (hring, &ofs);
		mesh_ring = InsertMesh (hring, 18, &ofs);
		SetMeshVisibilityMode (mesh_ring, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_ring;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddTMAVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_tma == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_tma = AddMesh (htma, &ofs);
		mesh_tma = InsertMesh (htma, 19, &ofs);
		SetMeshVisibilityMode (mesh_tma, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_tma;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		SetCameraOffset (_V(ofs.x-0,ofs.y+2.4,ofs.z+0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::AddSoyServiceVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_soyservice == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_soyservice = AddMesh (hsoyservice, &ofs);
		mesh_soyservice= InsertMesh (hsoyservice, 20, &ofs);
		SetMeshVisibilityMode (mesh_soyservice, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_soyservice;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void Soyuz::ClearMeshes ()
{
	VESSEL::ClearMeshes();
	mesh_booster1 = MESH_UNDEFINED;
	mesh_booster2 = MESH_UNDEFINED;
	mesh_booster3 = MESH_UNDEFINED;
	mesh_booster4 = MESH_UNDEFINED;
	mesh_stage1   = MESH_UNDEFINED;
	mesh_stage2   = MESH_UNDEFINED;
	mesh_fairtop  = MESH_UNDEFINED;
	mesh_fairbtm  = MESH_UNDEFINED;
	mesh_fair1    = MESH_UNDEFINED;
	mesh_fair2    = MESH_UNDEFINED;
	mesh_fairl    = MESH_UNDEFINED;
	mesh_fairr    = MESH_UNDEFINED;
	mesh_fairf    = MESH_UNDEFINED;
	mesh_fairb    = MESH_UNDEFINED;
	mesh_istg1    = MESH_UNDEFINED;
	mesh_istg2    = MESH_UNDEFINED;
	mesh_istg3    = MESH_UNDEFINED;
	mesh_escape   = MESH_UNDEFINED;
	mesh_ring     = MESH_UNDEFINED;
	mesh_tma      = MESH_UNDEFINED;
	mesh_soyservice = MESH_UNDEFINED;
	mesh_soyhs    = MESH_UNDEFINED;
}

void Soyuz::AddThrusters() {

	int i;
	VECTOR3 dir1, dir2, dir3, dir4;
	VECTOR3 loc1, loc2, loc3, loc4;

	PARTICLESTREAMSPEC exhaust_main = {
		0, 1.5, 1000.0, 100.0, 0.01, 0.5, 5, 0.1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
	};

	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	PARTICLESTREAMSPEC exhaust_second = {
		0, 1.0, 1000.0, 50, 0.1, 0.2 , 5, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
	};
	exhaust_second.tex = oapiRegisterParticleTexture ("Contrail2");

	PARTICLESTREAMSPEC rcs_exhaust = {
		0, 0.01, 1000, 2, 0, 0.05, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
	};
	rcs_exhaust.tex = oapiRegisterParticleTexture ("Contrail");
	PARTICLESTREAMSPEC oms_exhaust = {
		0, 0.1, 1000, 10, 0, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
	};
	oms_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");

	SURFHANDLE tex_main = oapiRegisterExhaustTexture ("Exhaust");

	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();


	if (ph_second==NULL)  
		ph_second  = CreatePropellantResource (FUEL_STG2); 
	
	switch( stage ) {
	case(0):
		for( i = 0 ; i < 4 ; i++ ) {
			if (ph_booster[i]==NULL)   {
				ph_booster[i] = CreatePropellantResource (FUEL_STG0); 
			}
		}
		
		if (ph_first==NULL)  
			ph_first  = CreatePropellantResource (FUEL_STG1); 
		
		th_booster[0] = CreateThruster (_V(3.2, 0.0, -10.5-comshift), _V(0,0,1),  952174.0, ph_booster[0], 2815.0, 0.0);
		th_booster[1] = CreateThruster (_V(-3.2, 0.0, -10.5-comshift), _V(0,0,1), 952174.0, ph_booster[1], 2815.0, 0.0);
		th_booster[2] = CreateThruster (_V(0, 3.2, -10.5-comshift), _V(0,0,1), 952174.0, ph_booster[2], 2815.0, 0.0);
		th_booster[3] = CreateThruster (_V(0, -3.2, -10.5-comshift), _V(0,0,1), 952174.0, ph_booster[3], 2815.0, 0.0);

		AddExhaustStream (th_booster[0], &exhaust_main);
		AddExhaustStream (th_booster[1], &exhaust_main);
		AddExhaustStream (th_booster[2], &exhaust_main);
		AddExhaustStream (th_booster[3], &exhaust_main);
		

	case(1):		

		if (ph_first==NULL)  
			ph_first  = CreatePropellantResource (FUEL_STG1); 
		
		th_first = CreateThruster (_V(0.0, 0.0, -11-comshift), _V(0,0,1), 1000000.0, ph_first, 2903.0, 0.0);

//		exhaust_main.v0 = 0.0; //50.0/0.8;
		AddExhaustStream (th_first, &exhaust_main);
//		AddExhaustStream (th_first, &exhaust_main2);
//		AddExhaust( th_first, 3.0, 0.5 );

		th_rleft[0] = CreateThruster (_V(-2.0, 0.0, -14.5-comshift), _V(0,-1,0), 50000.0, ph_first, 2903.0, 0.0);
		th_rleft[1] = CreateThruster (_V(2.0, 0.0, -14.5-comshift), _V(0,1,0), 50000.0, ph_first, 2903.0, 0.0);

		th_rright[0] = CreateThruster (_V(-2.0, 0.0, -14.5-comshift), _V(0,1,0), 50000.0, ph_first, 2903.0, 0.0);
		th_rright[1] = CreateThruster (_V(2.0, 0.0, -14.5-comshift), _V(0,-1,0), 50000.0, ph_first, 2903.0, 0.0);

		th_pup = CreateThruster (_V(0.0, 0.0, -14.5-comshift), _V(0,-1,0), 50000.0, ph_first, 2903.0, 0.0);
		th_pdown = CreateThruster (_V(0.0, 0.0, -14.5-comshift), _V(0,1,0), 50000.0, ph_first, 2903.0, 0.0);
		th_yleft = CreateThruster (_V(0.0, 0.0, -14.5-comshift), _V(1,0,0), 50000.0, ph_first, 2903.0, 0.0);
		th_yright = CreateThruster (_V(0.0, 0.0, -14.5-comshift), _V(-1,0,0), 50000.0, ph_first, 2903.0, 0.0);

		CreateThrusterGroup( &th_pup, 1, THGROUP_ATT_PITCHUP );
		CreateThrusterGroup( &th_pdown, 1, THGROUP_ATT_PITCHDOWN );
		CreateThrusterGroup( &th_yleft, 1, THGROUP_ATT_YAWLEFT );
		CreateThrusterGroup( &th_yright, 1, THGROUP_ATT_YAWRIGHT );

		CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
		CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );

		break;

	case ( 2 ):
	
		dir1 = _V(0,-.5,-1);
		dir2 = _V(-.5,0,-1);
		dir3 = _V(0,.5,-1);
		dir4 = _V(.5,0,-1);
		loc1 = _V(0, 1, 18-comshift);
		loc2 = _V(1, 0, 18-comshift);
		loc3 = _V(0,-1, 18-comshift);
		loc4 = _V(-1, 0, 18-comshift);
		th_second = CreateThruster (_V(0.0, 0.0, 17.5-comshift), _V(0,0,1), 290300.0, ph_second, 3865.0, 0.0);
		th_vern[0] = CreateThruster( loc1, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[1] = CreateThruster( loc2, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[2] = CreateThruster( loc3, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[3] = CreateThruster( loc4, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		AddExhaust (th_second, 4.5, .5, _V(0.0, 0.0, 17.5-comshift), _V(0,0,-1), tex_main);
		AddExhaust (th_vern[0], 4, .5, loc1, dir1, tex_main);
		AddExhaust (th_vern[1], 4, .5, loc2, dir2, tex_main);
		AddExhaust (th_vern[2], 4, .5, loc3, dir3, tex_main);
		AddExhaust (th_vern[3], 4, .5, loc4, dir4, tex_main);
		AddExhaustStream (th_second, &exhaust_second);
//		AddExhaustStream (th_second, &oms_exhaust);
		AddExhaustStream (th_vern[0], &oms_exhaust);
		AddExhaustStream (th_vern[1], &oms_exhaust);
		AddExhaustStream (th_vern[2], &oms_exhaust);
		AddExhaustStream (th_vern[3], &oms_exhaust);
		
		th_pup = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_pdown = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_yleft = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(1,0,0), 1000.0, ph_second, 2903.0, 0.0);
		th_yright = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(-1,0,0), 1000.0, ph_second, 2903.0, 0.0);

		th_rleft[0] = CreateThruster (_V(-2.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_rleft[1] = CreateThruster (_V(2.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);

		th_rright[0] = CreateThruster (_V(-2.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_rright[1] = CreateThruster (_V(2.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);

		CreateThrusterGroup( &th_pup, 1, THGROUP_ATT_PITCHUP );
		CreateThrusterGroup( &th_pdown, 1, THGROUP_ATT_PITCHDOWN );
		CreateThrusterGroup( &th_yleft, 1, THGROUP_ATT_YAWLEFT );
		CreateThrusterGroup( &th_yright, 1, THGROUP_ATT_YAWRIGHT );

		CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
		CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );

			
	default:

		dir1 = _V(0,-.5,-1);
		dir2 = _V(-.5,0,-1);
		dir3 = _V(0,.5,-1);
		dir4 = _V(.5,0,-1);
		loc1 = _V(0, 1, 18-comshift);
		loc2 = _V(1, 0, 18-comshift);
		loc3 = _V(0,-1, 18-comshift);
		loc4 = _V(-1, 0, 18-comshift);
		th_second = CreateThruster (_V(0.0, 0.0, 17.5-comshift), _V(0,0,1), 290300.0, ph_second, 3865.0, 0.0);
		th_vern[0] = CreateThruster( loc1, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[1] = CreateThruster( loc2, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[2] = CreateThruster( loc3, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		th_vern[3] = CreateThruster( loc4, _V(0,0,1), 1000, ph_second, 3865.0, 0.0);
		AddExhaust (th_second, 4.5, .5, _V(0.0, 0.0, 17.5-comshift), _V(0,0,-1), tex_main);
		AddExhaust (th_vern[0], 4, .5, loc1, dir1, tex_main);
		AddExhaust (th_vern[1], 4, .5, loc2, dir2, tex_main);
		AddExhaust (th_vern[2], 4, .5, loc3, dir3, tex_main);
		AddExhaust (th_vern[3], 4, .5, loc4, dir4, tex_main);
		AddExhaustStream (th_second, &exhaust_second);
//		AddExhaustStream (th_second, &oms_exhaust);
		AddExhaustStream (th_vern[0], &oms_exhaust);
		AddExhaustStream (th_vern[1], &oms_exhaust);
		AddExhaustStream (th_vern[2], &oms_exhaust);
		AddExhaustStream (th_vern[3], &oms_exhaust);
		
		th_pup = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_pdown = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_yleft = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(1,0,0), 1000.0, ph_second, 2903.0, 0.0);
		th_yright = CreateThruster (_V(0.0, 0.0, -5.0-comshift), _V(-1,0,0), 1000.0, ph_second, 2903.0, 0.0);

		th_rleft[0] = CreateThruster (_V(-2.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_rleft[1] = CreateThruster (_V(2.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);

		th_rright[0] = CreateThruster (_V(-2.0, 0.0, -5.0-comshift), _V(0,1,0), 1000.0, ph_second, 2903.0, 0.0);
		th_rright[1] = CreateThruster (_V(2.0, 0.0, -5.0-comshift), _V(0,-1,0), 1000.0, ph_second, 2903.0, 0.0);

		CreateThrusterGroup( &th_pup, 1, THGROUP_ATT_PITCHUP );
		CreateThrusterGroup( &th_pdown, 1, THGROUP_ATT_PITCHDOWN );
		CreateThrusterGroup( &th_yleft, 1, THGROUP_ATT_YAWLEFT );
		CreateThrusterGroup( &th_yright, 1, THGROUP_ATT_YAWRIGHT );

		CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
		CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );

			
	}

}

void Soyuz::JettisonBoosters()
{
	// Create SRB's as individual objects
	VESSELSTATUS2 vs;
	// VESSELSTATUS2::FUELSPEC fuel;
	// VESSELSTATUS2::THRUSTSPEC thrust;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {.5,.5,0};
	Local2Rel (OFF_BOOSTER[0]-_V(0.0,0.0,comshift), vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.y = -0.02;
	vs.vrot.x = 0.02;
	// vs.arot.z += 0.25*PI;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName()); strcat (name, "-B1");
	oapiCreateVesselEx (name, "r7blockb", &vs);
	vel = _V(-.5,.5,0);
	Local2Rel (OFF_BOOSTER[1]-_V(0.0,0.0,comshift), vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.y = 0.02;
	vs.vrot.x = 0.02;
	// vs.arot.z -= .75*PI;
	name[strlen(name)-1] = '2';
	oapiCreateVesselEx (name, "r7blockw", &vs);
	vel = _V(-.5,-.5,0);
	Local2Rel (OFF_BOOSTER[2]-_V(0.0,0.0,comshift), vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = -0.02;
	vs.vrot.y = 0.02;
	// vs.arot.z -= 1.25*PI;
	name[strlen(name)-1] = '3';
	oapiCreateVesselEx (name, "r7blockg", &vs);
	vel = _V(.5,-.5,0);
	Local2Rel (OFF_BOOSTER[3]-_V(0.0,0.0,comshift), vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = -0.02;
	vs.vrot.y = -0.02;
	// vs.arot.z -= 1.75*PI;
	name[strlen(name)-1] = '4';
	oapiCreateVesselEx (name, "r7blockd", &vs);

//	VESSELSTATUS vs[4];
//	VECTOR3 rvel, tmp;

//	int i;
//	for( i = 0 ; i < 4 ; i++ ) {
//		GetStatus (vs[i]);
//		vs[i].eng_main = vs[i].eng_hovr = 0.0;
//		Local2Rel (OFF_BOOSTER[i]-_V(0.0,0.0,comshift), vs[i].rpos);
//
//		tmp = OFF_BOOSTER[i];
//		tmp.z = 0.0;
//		rvel.x = 10.5 * tmp.x / mod(&tmp);
//		rvel.x = 10.5 * tmp.x - int(10.5 * tmp.x);
//		rvel.y = 10.5 * tmp.y / mod(&tmp);
//		rvel.y = 10.5 * tmp.y - int(10.5 * tmp.y);
//		rvel.z = 0.0;
//		GlobalRot( rvel, tmp );
//		vs[i].rvel += tmp;
//	}

//	vs[0].vrot.y += -0.25;
//	vs[1].vrot.x += 0.18;
//	vs[2].vrot.y += 0.21;		
//	vs[3].vrot.x += -0.12;		

	DelMesh (mesh_booster1);
	DelMesh (mesh_booster2);
	DelMesh (mesh_booster3);
	DelMesh (mesh_booster4);
//	oapiDeleteMesh (hbooster[0]);
//	oapiDeleteMesh (hbooster[1]);
//	oapiDeleteMesh (hbooster[2]);
//	oapiDeleteMesh (hbooster[3]);

//	oapiCreateVessel("Strap-on Booster B", "r7blockb", vs[0]);
//	oapiCreateVessel("Strap-on Booster W", "r7blockw", vs[1]);
//	oapiCreateVessel("Strap-on Booster D", "r7blockd", vs[2]);
//	oapiCreateVessel("Strap-on Booster G", "r7blockg", vs[3]);

	

}
void Soyuz::JettisonStg1()
{
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = OFF_STG1-_V(0,0,22.2);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (OFF_STG1-_V(0,0,22.2));
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,-1};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = -0.02;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-Stg1");
	oapiCreateVesselEx (name, "r7blocka", &vs);

//	VESSELSTATUS vs;

//	GetStatus (vs);
//	vs.eng_main = vs.eng_hovr = 0.0;

//	Local2Rel(OFF_STG1-_V(0.0,0.0,comshift), vs.rpos);
   
//	oapiCreateVessel  ("soyuz-stg1", "r7blocka", vs);

	DelMesh (mesh_stage1);
//	oapiDeleteMesh (hstage1);

}
void Soyuz::JettisonFairing()
{

	if( !fair_attached )
		return;
	
	PARTICLESTREAMSPEC escape_exhaust = {
		0, 1.5, 1000.0, 100.0, 0.01, 0.5, 5, 0.1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
	};

	escape_exhaust.tex = oapiRegisterParticleTexture ("Contrail");
	
	
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
//	VECTOR3 ofs = OFF_LHF-_V(0.0,0.0,comshift);
	VECTOR3 ofs = OFF_LHF;
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= OFF_LHF;
	VECTOR3 lvel;
	VECTOR3 rot;
	VECTOR3 rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	Local2Rel(ofs, vs.rpos);
	if( manned_launch ) {
		lvel = _V( 0, 10, 0 );
		rot = _V(  0.5, 0, 0 );
	}
	else {
		lvel = _V( -10, 0, 0 );
		rot = _V( 0, 0.5, 0 );
	}
	GlobalRot( lvel, rvel );
	vs.rvel.x += rvel.x;
	vs.rvel.y += rvel.y;
	vs.rvel.z += rvel.z;
	vs.vrot = rot;
	vs.status = 0;
	char name[256];

	if( manned_launch ) {
		strcpy (name, GetName());  strcat (name, "-Shd1");
		oapiCreateVesselEx(name, "soyshrd1", &vs);
	}
	else{
		strcpy (name, GetName());  strcat (name, "-Frg1");
		oapiCreateVesselEx(name, "soyufair1", &vs);
	}



	lvel.x *= -1;
	lvel.y *= -1;
	rot.x *= -1;
	rot.y *= -1;

	GlobalRot( lvel, rvel );

	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
//	ofs = OFF_RHF-_V(0.0,0.0,comshift);
	ofs = OFF_RHF;
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= OFF_RHF;

	vs.rvel.x += rvel.x;
	vs.rvel.y += rvel.y;
	vs.rvel.z += rvel.z;

	vs.vrot = rot;

	Local2Rel(ofs, vs.rpos);
   

	if( manned_launch ) {
		strcpy (name, GetName());  strcat (name, "-Shd2");
		oapiCreateVesselEx(name, "soyshrd2", &vs);
	}
	else{
		strcpy (name, GetName());  strcat (name, "-Frg2");
		oapiCreateVesselEx(name, "soyufair2", &vs);
	}
	DelMesh (mesh_fairtop);
	DelMesh (mesh_fairbtm);
//	oapiDeleteMesh (hfairtop);
//	oapiDeleteMesh (hfairbtm);

	
	if( manned_launch ) {
		lvel = _V( 0, 0, 2.0 );
		rot = _V(  0, -0.1, 0 );

		GlobalRot( lvel, rvel );

//		ofs = OFF_ESCAPE-_V(0.0,0.0,comshift);
		ofs = OFF_ESCAPE;

		GetStatusEx (&vs);
		vs.flag = VS_FUELRESET | VS_THRUSTRESET;

		vs.rvel.x += rvel.x;
		vs.rvel.y += rvel.y;
		vs.rvel.z += rvel.z;

		vs.vrot = rot;

		Local2Rel(ofs, vs.rpos);
   
		strcpy (name, GetName());  strcat (name, "-TMAER");
		oapiCreateVesselEx(name, "r7tmaer", &vs);

		DelMesh (mesh_escape);
//		oapiDeleteMesh (hescape);

	}




	fair_attached = false;

	Refresh();
	
}

void Soyuz::JettisonPayload ()
{
	// Create Tank as individual object
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs;
	if (stage > 1 && stage < 4 || stage > 11)
		ofs = OFF_FLIGHT_TMA;
	else {
		if (stage < 2 || stage > 9 && stage < 12)
			ofs = OFF_TMA;
	}
	if (Playback() && (stage > 1 && stage < 4 || stage > 11)) // necessary because during playback the CG shift occurs before separation
		ofs -= OFF_FLIGHT_TMA;
	else {
		if (Playback() && (stage < 2 || stage > 9 && stage < 12))
			ofs -= OFF_TMA;
	}
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-TMA");
	ohtma = oapiCreateVesselEx (name, "SoyuzTMA", &vs);

	// remove tank mesh and shift cg

}

void Soyuz::JettisonEscape ()
{
	// Create Tank as individual object
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = OFF_BHF + OFF_ESC;
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= OFF_BHF + OFF_ESC;
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-TMA");
	ohesc = oapiCreateVesselEx (name, "SoyuzDES", &vs);

	// remove tank mesh and shift cg

}

void Soyuz::OperateAbort (DoorStatus action)
{

	abort_status = action;
	RecordEvent ("ABORT", action == DOOR_CLOSING ? "Stored" : "Deployed");
}

void Soyuz::RevertAbort ()
{

	OperateAbort (abort_status == DOOR_CLOSED || abort_status == DOOR_CLOSING ?
		DOOR_OPENING : DOOR_CLOSING);
}

// --------------------------------------------------------------
// Define animation sequences for moving parts
// --------------------------------------------------------------
void Soyuz::DefineAnimations (void)
{
	// ***** 1. Solar Panel animation *****

	// 0.5 radians = 90 degrees - 0.0056 radians = 1 degree

	// 0.1111 radians = 20 degrees

	static UINT RightGrp[1] = {17};
	static MGROUP_ROTATE RightPanel (mesh_fairtop, RightGrp, 1,
		_V(-0.945,0.946,-2.656), _V(-.5,-.5,0), 90*RAD);
	static UINT FrontGrp[1] = {19};
	static MGROUP_ROTATE FrontPanel (mesh_fairtop, FrontGrp, 1,
		_V(-0.946,-0.947,-2.656), _V(-.5,.5,0), -90*RAD);
	static UINT LeftGrp[1] = {18};
	static MGROUP_ROTATE LeftPanel (mesh_fairtop, LeftGrp, 1,
		_V(0.944,-0.952,-2.656), _V(.5,.5,0), 90*RAD);
	static UINT RearGrp[1] = {16};
	static MGROUP_ROTATE RearPanel (mesh_fairtop, RearGrp, 1,
		_V(0.946,0.946,-2.656), _V(.5,-.5,0), -90*RAD);

	anim_abort = CreateAnimation (0);

	AddAnimationComponent (anim_abort, 0.0, 1.00, &RightPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &LeftPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &FrontPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &RearPanel);


}
void Soyuz::UpdateMesh ()
{
	// update animation states
	if ( fair_attached ) {
		SetAnimation (anim_abort, abort_proc);
	}

}

// ==============================================================
// Overloaded callback functions
// ==============================================================

void Soyuz::clbkSetClassCaps(  FILEHANDLE cfg )
{

	if (!oapiReadItem_bool (cfg, "RenderCockpit", render_cockpit))
		render_cockpit = false;

}

int Soyuz::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events


		switch (key) {
		case OAPI_KEY_J:  // "Jettison"
			if (!Playback()) {
				manual_separate = true;
			}
			return 1;
		case OAPI_KEY_F:  // "Jettison"
			if (!Playback()) {
				JettisonFairing();
			}
			return 1;
		case OAPI_KEY_A:  // "Solar Panels"
			if (!KEYMOD_SHIFT(kstate)) {
				if (!Playback()&& fair_attached) 
					RevertAbort ();
				if (!Playback()&& !fair_attached) 
					abort_status = DOOR_OPEN;
			}
			return 1;
		}

	return 0;
}

void Soyuz::clbkPostStep(  double simt, double simdt, double mjd )
{
//	sprintf( oapiDebugString(), "%2.2f", GetMass() );

	if ( fair_attached ) {
		if (abort_status >= DOOR_CLOSING) {
			double da = simdt * ABORT_OPERATING_SPEED;
			if (abort_status == DOOR_CLOSING) { // retract gear
				if (abort_proc > 0.0) abort_proc = max (0.0, abort_proc-da);
				else  {
					abort_status = DOOR_CLOSED;
				}
			} else {                           // deploy gear
				if (abort_proc < 1.0) {
					abort_proc = min (1.0, abort_proc+da);  //Door Opening
				}
				else  {
					abort_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_abort, abort_proc);
		}
	}
	
//	if ( abort_status == DOOR_OPEN && fair_attached && stage < 3) {
	if ( abort_status == DOOR_OPEN && stage < 3) {
		SetThrusterGroupLevel( THGROUP_MAIN, 0.0 );
		AbortLaunch();
		if (stage == 10 || stage == 11 && fair_attached) {
			JettisonEscape();
			oapiSetFocusObject(ohesc);
		}
		if (stage == 11 && !fair_attached || stage > 11) {
			JettisonPayload();
			oapiSetFocusObject(ohtma);
		}
	}

//	if ( abort_status == DOOR_OPENING && fair_attached) {
//		SetAbortStage();
//	}

	if( fair_attached && GetAltitude() > 100000.0 && stage < 10 ) {
		JettisonFairing();
	}

	if( stage == 0 ) {
		if( manual_separate || GetPropellantMass( ph_booster[0] ) < 1000.0 )
		{
			manual_separate=false;
			JettisonBoosters();
			stage = 1;
			Refresh();
			SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		}
	}
	else if( stage == 1 ) {
		if( manual_separate || GetPropellantMass( ph_first ) < 1000.0 )
		{
			manual_separate=false;
			JettisonStg1();
			stage = 2;
			Refresh();
			SetThrusterGroupLevel(THGROUP_MAIN,1.0);
		}
	}
	else if( stage == 2 ) {
		if( manual_separate && !fair_attached ) {
			manual_separate = false;
			SetPayloadStage();
			JettisonPayload();
			oapiSetFocusObject(ohtma);
			Refresh();
		}
	}


}

void Soyuz::clbkSetStateEx (const void *status)
{
	// default parameter initialisation
	DefSetStateEx (status);

	// reset vessel-specific parameters to defaults
	stage = 0;
	Refresh();
	DefineAnimations();
	UpdateMesh();

}

void Soyuz::clbkSaveState( FILEHANDLE scn )
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage );
	if( fair_attached )
		oapiWriteScenario_string (scn, "FAIRING", "1" );
	else
		oapiWriteScenario_string (scn, "FAIRING", "0" );
	if( manned_launch )
		oapiWriteScenario_string (scn, "MANNED", "1" );
	else
		oapiWriteScenario_string (scn, "MANNED", "0" );
	sprintf (cbuf, "%d %0.4f", abort_status, abort_proc);
	oapiWriteScenario_string (scn, "ABORT", cbuf);

}

void Soyuz::clbkLoadStateEx(  FILEHANDLE scn, void *vs )
{
    char *line;
	int manned_in;
	int fairing_in;
	fair_attached = false;
	manned_launch = false;

	ph_booster[0]	= NULL;	
	ph_booster[1]	= NULL;	
	ph_booster[2]	= NULL;	
	ph_booster[3]	= NULL;	
	ph_first        = NULL;
	ph_second       = NULL;

	ClearMeshes();
	while (oapiReadScenario_nextline (scn, line)) 
	{
	  if (!strnicmp (line, "CONFIGURATION", 13)) {
				sscanf (line+13, "%d", &stage);
		} else if( !strnicmp( line, "FAIRING", 7 )) {
			sscanf (line+7, "%d", &fairing_in);
			if (fairing_in == 1)
				fair_attached = true;
		}
		else if( !strnicmp( line, "MANNED", 6 )) {
			sscanf (line+6, "%d", &manned_in);
			if (manned_in == 1)
				manned_launch = true;
		}
		else if (!strnicmp (line, "ABORT", 5)) {
			sscanf (line+5, "%d%lf", &abort_status, &abort_proc);
		}
		else 
		{
			ParseScenarioLineEx (line, vs);
		}
	}

	bool Abort_Launch;
	Abort_Launch = false;
	if (stage > 3) {
		Abort_Launch = true;
		stage = stage - 10;
	}
	ClearMeshes();
	Refresh();
	DefineAnimations();
	UpdateMesh();
	if (Abort_Launch) {
		AbortLaunch();
	}
}

void Soyuz::clbkFocusChanged (bool getfocus, OBJHANDLE newv, OBJHANDLE oldv)
{
	if (getfocus) {
		oapiDisableMFDMode (MFD_LANDING);
		// no VTOL MFD mode for Atlantis
	}
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}

bool Soyuz::clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event)
{

	return false;

}

void Soyuz::clbkVisualCreated (VISHANDLE _vis, int refcount)
{

	if (refcount > 1) return; // we don't support more than one visual per object
	vis = _vis;

	//UpdateMesh ();
}

void Soyuz::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{

	if (vis == _vis) vis = NULL;

}

void Soyuz::clbkAnimate (double simt)
{

	UpdateMesh ();

}

bool Soyuz::clbkLoadVC (int id)
{

	switch (id) {
	case 0:
		oapiVCRegisterHUD (&huds);
		oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
		oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
		return true;
	default:
		return false;
	}

}
// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Module initialisation
// --------------------------------------------------------------
DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Soyuz (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (Soyuz*)vessel;
}

void Soyuz::Refresh() {

	if( manned_launch ) {
		hfair1 = hfairl;
		hfair2 = hfairr;
		if (stage < 2) {
			OFF_THF = _V(0,  0, 32.251);
			OFF_BHF = _V(0,  0, 27.665);
			OFF_LHF = _V(0,  0.8, 31.2);
			OFF_RHF = _V(0, -0.8, 31.2);
		} else {
			OFF_THF = _V(0,  0,10.051);
			OFF_BHF = _V(0,  0, 5.465);
			OFF_LHF = _V(0,  0.8, 9.0);
			OFF_RHF = _V(0, -0.8, 9.0);
		}
	}
	else {
		hfair1 = hfairf;
		hfair2 = hfairb;
		if (stage < 2) {
			OFF_LHF = _V(0.0,  0, 27);
			OFF_RHF = _V(0.0,  0, 27);
		} else {
			OFF_LHF = _V(0.0,  0, 4.8);
			OFF_RHF = _V(0.0,  0, 4.8);
		}
	}

	switch( stage ) {
	case( 0 ):
		SetBoostStage();
		break;
	case( 1 ):
		SetMainStage();
		break;
	case ( 2 ):
		SetSecondStage();
		break;
	case ( 3 ):
		SetPayloadStage();
		break;
	}

}