// ==============================================================
// $Id: soyuz.h,v 1.6 2004/09/07 00:35:29 dh219 Exp $
// Soyuz launcher by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================
#ifndef __Soyuz_H
#define __Soyuz_H

#include "orbitersdk.h"
#include <stdio.h>
#include <math.h>

const DOUBLE MASS_STG0 = 15240.0; // (3810.0*4.0)
const DOUBLE MASS_STG1 = 6875.0;
const DOUBLE MASS_STG2 = 2355.0;
const DOUBLE MASS_TMA = 7220.0;

const DOUBLE FUEL_STG0 = 40590.0;
const DOUBLE FUEL_STG1 = 98525.0;
const DOUBLE FUEL_STG2 = 22845.0;

const DOUBLE BLOCKA_MASS = 6800.0;
const DOUBLE BLOCKD_MASS = 3710.0;
const DOUBLE BLOCKW_MASS = 3710.0;
const DOUBLE BLOCKG_MASS = 3710.0;
const DOUBLE BLOCKB_MASS = 3710.0;
const DOUBLE SHROUD1_MASS = 1400.0;
const DOUBLE SHROUD2_MASS = 1400.0;
const DOUBLE FAIR1_MASS = 1400.0;
const DOUBLE FAIR2_MASS = 1400.0;
const DOUBLE R7TMAER_MASS = 1000.0;

const UINT MESH_UNDEFINED = (UINT)-1;
const double ABORT_OPERATING_SPEED = 10;

class Soyuz: public VESSEL2 {

public:
	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } abort_status;
	Soyuz (OBJHANDLE hObj, int fmodel);
	~Soyuz ();

	void Refresh();

	double abort_proc; // abort latch deployment state (0=retracted, 1=deployed)

	VISHANDLE vis;

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkSetStateEx (const void *status);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	void clbkPostStep (double simt, double simdt, double mjd);
	bool clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkAnimate (double simt);
	bool clbkLoadVC (int id);

private:
	void AbortLaunch ();
	void AddBooster1Visual (const VECTOR3 &ofs);
	void AddBooster2Visual (const VECTOR3 &ofs);
	void AddBooster3Visual (const VECTOR3 &ofs);
	void AddBooster4Visual (const VECTOR3 &ofs);
	void AddStage1Visual (const VECTOR3 &ofs);
	void AddStage2Visual (const VECTOR3 &ofs);
	void AddFairingtopVisual (const VECTOR3 &ofs);
	void AddFairingbtmVisual (const VECTOR3 &ofs);
	void AddFairing1Visual (const VECTOR3 &ofs);
	void AddFairing2Visual (const VECTOR3 &ofs);
	void AddFairinglVisual (const VECTOR3 &ofs);
	void AddFairingrVisual (const VECTOR3 &ofs);
	void AddFairingfVisual (const VECTOR3 &ofs);
	void AddFairingbVisual (const VECTOR3 &ofs);
	void AddIstg1Visual (const VECTOR3 &ofs);
	void AddIstg2Visual (const VECTOR3 &ofs);
	void AddIstg3Visual (const VECTOR3 &ofs);
	void AddRingVisual (const VECTOR3 &ofs);
	void AddEscapeVisual (const VECTOR3 &ofs);
	void AddSoyServiceVisual (const VECTOR3 &ofs);
	void AddTMAVisual (const VECTOR3 &ofs);
	void DefineAnimations (void);
	void OperateAbort (DoorStatus action);
	void RevertAbort ();
	void SetAbortStage();
	void SetBoostStage();
	void SetMainStage ();
	void SetSecondStage ();
	void SetPayloadStage ();
	void JettisonStg1();
	void JettisonBoosters();
	void JettisonEscape();
	void JettisonFairing();
	void JettisonPayload();
	void AddThrusters();
	void UpdateMesh ();
	void ClearMeshes ();

	PROPELLANT_HANDLE ph_booster[4];	
	PROPELLANT_HANDLE ph_first;	
	PROPELLANT_HANDLE ph_second;
	THRUSTER_HANDLE th_booster[4];
	THRUSTER_HANDLE th_first;
	THRUSTER_HANDLE th_second;
	THRUSTER_HANDLE th_pup;
	THRUSTER_HANDLE th_pdown;
	THRUSTER_HANDLE th_yleft;
	THRUSTER_HANDLE th_yright;
	THRUSTER_HANDLE th_rleft[2];
	THRUSTER_HANDLE th_rright[2];
	THRUSTER_HANDLE th_main[5];
	THRUSTER_HANDLE th_vern[4];

	MESHHANDLE hbooster[4];
	MESHHANDLE hstage1;
	MESHHANDLE hstage2;
	MESHHANDLE hfairtop;
	MESHHANDLE hfairbtm;
	MESHHANDLE hfairl;
	MESHHANDLE hfairr;
	MESHHANDLE hfairf;
	MESHHANDLE hfairb;
	MESHHANDLE histg[3];
	MESHHANDLE hfair1;
	MESHHANDLE hfair2;
	MESHHANDLE hescape;
	MESHHANDLE hring;
	MESHHANDLE htma;
	MESHHANDLE hsoyorbit;
	MESHHANDLE hsoyservice;
	MESHHANDLE hsoydescent;
	MESHHANDLE hsoyhs;

	OBJHANDLE ohtma;
	OBJHANDLE ohesc;

	UINT mesh_booster1;
	UINT mesh_booster2;
	UINT mesh_booster3;
	UINT mesh_booster4;
	UINT mesh_stage1;
	UINT mesh_stage2;
	UINT mesh_fairtop;
	UINT mesh_fairbtm;
	UINT mesh_fairl;
	UINT mesh_fairr;
	UINT mesh_fairf;
	UINT mesh_fairb;
	UINT mesh_fair1;
	UINT mesh_fair2;
	UINT mesh_istg1;
	UINT mesh_istg2;
	UINT mesh_istg3;
	UINT mesh_escape;
	UINT mesh_ring;
	UINT mesh_tma;
	UINT anim_abort;                              // handle for landing gear animation
	UINT mesh_soyorbit;
	UINT mesh_soyservice;
	UINT mesh_soydescent;
	UINT mesh_soyhs;
	
	bool fair_attached;
	bool manned_launch;

	double comshift;
	bool manual_separate;
	SURFHANDLE tex;

	int stage;

	VECTOR3 OFF_BOOSTER[4];
	VECTOR3 OFF_STG1;
	VECTOR3 OFF_STG2;
	VECTOR3 OFF_LHF;
	VECTOR3 OFF_RHF;
	VECTOR3 OFF_THF;
	VECTOR3 OFF_BHF;
	VECTOR3 OFF_ISTG[3];
	VECTOR3 OFF_ESCAPE;
	VECTOR3 OFF_SERVICE;
	VECTOR3 OFF_FLIGHT_ESCAPE;
	VECTOR3 OFF_RING;
	VECTOR3 OFF_TMA;
	VECTOR3 OFF_FLIGHT_STG2;
	VECTOR3 OFF_FLIGHT_ISTG1;
	VECTOR3 OFF_FLIGHT_ISTG2;
	VECTOR3 OFF_FLIGHT_ISTG3;
	VECTOR3 OFF_FLIGHT_TMA;
	VECTOR3 OFF_ZERO;
	VECTOR3 OFF_ESC;
	VECTOR3 OFF_FLIGHT_RING;
	VECTOR3 OFF_FLIGHT_ORBIT;
	VECTOR3 OFF_FLIGHT_SERVICE;

	bool render_cockpit;
	VCHUDSPEC huds;
	VCMFDSPEC mfds_left;
	VCMFDSPEC mfds_right;

};

class r7blockb: public VESSEL2 {
public:
	r7blockb (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hblockbmesh;
	UINT mesh_blockb;                              // index for Inter-Stage mesh
};

class r7blockw: public VESSEL2 {
public:
	r7blockw (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hblockwmesh;
	UINT mesh_blockw;                              // index for Inter-Stage mesh
};

class r7blockd: public VESSEL2 {
public:
	r7blockd (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hblockdmesh;
	UINT mesh_blockd;                              // index for Inter-Stage mesh
};

class r7blockg: public VESSEL2 {
public:
	r7blockg (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hblockgmesh;
	UINT mesh_blockg;                              // index for Inter-Stage mesh
};

class r7blocka: public VESSEL2 {
public:
	r7blocka (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hblockamesh;
	UINT mesh_blocka;                              // index for Inter-Stage mesh
};

class soyshrd1: public VESSEL2 {
public:
	soyshrd1 (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hsoyshrd1mesh;
	UINT mesh_soyshrd1;                              // index for Inter-Stage mesh
};

class soyufair1: public VESSEL2 {
public:
	soyufair1 (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hsoyufair1mesh;
	UINT mesh_soyufair1;                              // index for Inter-Stage mesh
};

class soyshrd2: public VESSEL2 {
public:
	soyshrd2 (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hsoyshrd2mesh;
	UINT mesh_soyshrd2;                              // index for Inter-Stage mesh
};

class soyufair2: public VESSEL2 {
public:
	soyufair2 (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hsoyufair2mesh;
	UINT mesh_soyufair2;                              // index for Inter-Stage mesh
};

class r7tmaer: public VESSEL2 {
public:
	r7tmaer (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation ();
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hr7tmaermesh;
	UINT mesh_r7tmaer;                              // index for Inter-Stage mesh
	PROPELLANT_HANDLE ph_escape;
	THRUSTER_HANDLE th_escape;
	THGROUP_HANDLE thg_ER;                   // handles for thruster groups
};

#endif

