// ==============================================================
// $Id: soyuzdes.cpp,v 1.11 2006/03/18 21:06:20 dh219 Exp $
// Soyuz TMA by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <stdio.h>
#include <fstream>
#include "SoyuzDES.h"

PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.05, 3000, 5, 0.1, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.4, 3000, 150, 0.05, 0.5, 5, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust2 = {
0, 0.4, 3000, 50, 0.05, 0.1, 5.0/3.0, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};

void CoeffFunc (double aoa, double M, double Re, double *cl, double *cm, double *cd)

{
	const int nlift = 11;
	double factor,dfact,lfact,frac,drag,lift;
	static const double AOA[nlift] =
		{-180.*RAD,-160.*RAD,-150.*RAD,-120.*RAD,-90.*RAD,0*RAD,90.*RAD,120.*RAD,150.*RAD,160.*RAD,180.*RAD};
	static const double Mach[17] = {0.0,0.7,0.9,1.1,1.2,1.35,1.65,2.0,3.0,5.0,8.0,10.5,13.5,18.2,21.5,31.0,50.0};
	static const double LFactor[17] = {0.3,0.392,0.466,0.607,0.641,0.488,0.446,0.435,0.416,0.415,0.405,0.400,0.385,0.385,0.375,0.35,0.33};
	static const double DFactor[17] = {0.9,0.944,0.991,1.068,1.044,1.270,1.28,1.267,1.213,1.134,1.15,1.158,1.8,1.8,1.193,1.224,1.25};
	static const double CL[nlift]  = {0.0,-0.9,-1.1,-0.5,0.0,0.0,0.0,0.5,1.1,0.9,0.0};
	static const double CM[nlift]  = {0.0,0.004,0.006,0.012,0.015,0.0,-0.015,-0.012,-0.006,-0.004,0.};
	static const double CD[nlift]  = {1.143,1.0,1.0,0.8,0.8,0.8,0.8,0.8,1.0,1.0,1.143};
	int j;
	factor = 0.0;
	dfact = 0.9;
	lfact = 1.0;
	for(j = 0; (j < 16) && (Mach[j+1] < M); j++);
	frac = (M-Mach[j])/(Mach[j+1]-Mach[j]);
	drag = dfact*(frac*DFactor[j+1]+(1.0-frac)*DFactor[j]);
	lift = drag * lfact*(frac*LFactor[j+1]+(1.0-frac)*LFactor[j]);
	for(j = 0; (j < nlift-1) && (AOA[j+1] < aoa); j++);
	frac = (aoa-AOA[j])/(AOA[j+1]-AOA[j]);
	*cd = drag*(frac*CD[j+1]+(1.0-frac)*CD[j]);
	*cl = lift*(frac*CL[j+1]+(1.0-frac)*CL[j]);
	*cm = factor*(frac*CM[j+1]+(1.0-frac)*CM[j]);
}

struct {
	HINSTANCE hDLL;
} g_Param;

SoyuzDES::SoyuzDES (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj)
{
	ph_first = NULL;
	ph_second = NULL;
	ph_srm = NULL;

	htma = oapiLoadMesh("SoyuzESC");
	hentry = oapiLoadMesh("soyuz-descent");
	hpara = oapiLoadMesh("soychute");
	hdrogue = oapiLoadMesh("soydrogue");
	hHSmesh = oapiLoadMesh("ACRMHS");
	DefineAnimations();
	abort_status = DOOR_OPENING;
	abort_proc = 0.50;
	i_state = 1.00;

	mesh_tma             = MESH_UNDEFINED;
	mesh_para            = MESH_UNDEFINED;
	mesh_drogue          = MESH_UNDEFINED;
	mesh_desc            = MESH_UNDEFINED;
	mesh_HS              = MESH_UNDEFINED;

	VZERO = _V(0,0,0);

	status = 0;
	shieldstatus = 0;
	targetstatus = 0;

	comshift = _V(0,0,0);
	OFS_DES = _V(0,0,-5.18);
	OFS_HS = _V(0,0,-6.06);

	chute = NULL;
	chute_timer = 3.0;
	start_time = 0.0;

	reenteractive = false;
	reentercoasting = false;
	target = 6445000.0;
	bManualSeparate = false;

	refresh = 0;

	dh=NULL;

}




SoyuzDES::~SoyuzDES()
{
	delete th_first;
}
//void SoyuzDES::TimeStep(  double simt )
//{
//	/* CVE-lite function ! */
//	/* IMPORTANT */
//	/* This function loops through the pending GPDS requests and returns a bool (true/false)
//	/* if any GPDS requests have been implemented.
//	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
//	/* call to the (repeatable) SetMainStage(); */

//	if( CheckCentralMessage() ) {
//		while( CheckCentralMessage() ) ;
//		SetMainStage();
//	}



//}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void SoyuzDES::SetMainStage( )
{

//	SetAnimation( anim3, 0.0 );

	VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	
	/* erase all meshes and thrusters so we can start afresh */
//	ClearMeshes();

	SetSize (15);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
//	SetEmptyMass ( MASS + CalculateMass() );
	SetEmptyMass ( MASS );

	ShiftCG(VZERO-comshift);
	comshift = _V(0,0,0);
	ShiftCG(comshift);


	SetTouchdownPoints (_V(0,-2,4)-comshift,_V(2,-2,-4)-comshift,_V(-2,-2,-4)-comshift);

	SetPMI (_V(5,5,0.85));
	SetCrossSections (_V(15,32,6));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.1,0.1,0.01));
	SetPitchMomentScale (-3e-6);
	SetBankMomentScale (3e-6);
	SetLiftCoeffFunc(0); 

	SetSurfaceFrictionCoeff (0.05, 0.5);
	SetCameraOffset( _V( 0.0, -1.50, 1.5)-comshift );


	VECTOR_ZERO = (comshift*-1.0);

	/* we're not using COMMON_MESH today, so draw our own */
	AddTMAVisual (VECTOR_ZERO);
	AddDescVisual (OFS_DES);
	AddHSVisual (OFS_HS);
//	UINT mshid = AddMesh("soyuz", &VECTOR_ZERO );
//	SetMeshVisibilityMode( mshid, MESHVIS_ALWAYS );

//	MeshVisibilityPreference(MESHVIS_ALWAYS);


	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */
	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
//	Draw(comshift*-1.0);

	
	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if( ph_first == NULL ) {
		ph_first = CreatePropellantResource(FUEL);
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}

	if( !dh )
		dh = CreateDock(_V(0,0,4.615),_V(0,0,1),_V(0,1,0));

	SetDefaultPropellantResource (ph_first); 


	AddThrusters();
	EnableTransponder(true);
	status = 1;
}


void SoyuzDES::SetEntryStage() {


	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,-0.5);
	ShiftCG(comshift);
	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );

	VECTOR3 off;
//	UINT msh;
	off = (_V(0,-0.1,-5.18));


	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
//	ClearMeshes();	

	SetSize (5);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

	SetPMI (ENTRY_PMI);
	SetCrossSections (REENTRY_CS);
	SetCW (REENTRY_CW[0],REENTRY_CW[1],REENTRY_CW[2],REENTRY_CW[3]);
	CreateAirfoil(LIFT_VERTICAL, _V(0.0, 0.05, 0.94), CoeffFunc, 2.24, 3.94, 1.0);

	SetRotDrag (REENTRY_RD);
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}
	
	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}

	if( ph_second && GetPropellantMass(ph_second) > FUELRCS ) {
		SetPropellantMass(ph_second, FUELRCS);
	}

	if (GetPropellantMass(ph_second) < FUELRCS) {
		SetPropellantMass(ph_second, FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

	status = 2;

}

void SoyuzDES::SetDrogueStage() {


	ClearAirfoilDefinitions ();
	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,0);
	ShiftCG(comshift);

//	SetPMI (ENTRY_PMI);
//	SetCrossSections (ENTRY_CS);
//	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

//	SetRotDrag (ENTRY_RD);
	SetPMI (_V(2,2,0.55));
	SetCrossSections (_V(5,5,40));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.05,0.05,0.05));
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);

	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );

	
	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
//	ClearMeshes();	

	SetSize (80);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	VECTOR3 off, off2;
//	UINT msh, msh2;
	off = (VZERO-comshift);
	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
	off2 = (VZERO-comshift)+_V(0,0,72);
	AddDrogueVisual (off2);
//	msh2 = AddMesh(hdrogue,&off2);
//	SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

/*
	if( animcomp1 != NULL )
		DelAnimationComponent(anim1, animcomp1 );
	if( animcomp2 != NULL )
		DelAnimationComponent(anim2, animcomp2 );
*/

/*	static UINT groups0[5] = { 0, 1, 2, 3, 4 };
	
	animcomp1 = AddAnimationComponent( anim1, 0, 1,
		new MGROUP_ROTATE (
			msh2,
			groups0, 5,
			_V(0,0,1)-off2,
			_V(1,0,0),
			30*RAD
		)
	);

	animcomp2 = AddAnimationComponent( anim2, 0, 1,
		new MGROUP_ROTATE (
			msh2,
			groups0, 5,
			_V(0,0,1)-off2,
			_V(0,1,0),
			30*RAD
		)
	);
*/
	status = 3;

}

void SoyuzDES::SetParaStage() {
	

//	Sets up Main parachute deployment

//	removes animation components
//	technique used in earlier version of orbiter but still supported
	
//	Removes arfoil definitions to allow SetCrossSections to work as Main Chute.
//	Another method to use is a different airfoil definition
	ClearAirfoilDefinitions ();

	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,0);
	ShiftCG(comshift);

//	SetPMI (ENTRY_PMI);
//	SetCrossSections (ENTRY_CS);
//	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

//	SetRotDrag (ENTRY_RD);

//	Sets parameters to emulate a main chute.
	SetPMI (_V(2,2,0.55));
	SetCrossSections (_V(5,5,1000));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.05,0.05,0.05));
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);


//	comshift = _V(0,-0.18,-1.265);

	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );
	
	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
//	ClearMeshes();	

	SetSize (80);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

//	If low enough in the atmosphere, Main engine and RCS not needed
//	better to deconfigure Thruster Definitions only, but
//	have not tested what additional propellant weight will do to this section.
	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	VECTOR3 off, off2;
//	UINT msh, msh2;
	off2 = (VZERO-comshift)+_V(0,0,72);
//	Add Parachute mesh
	AddParaVisual (off2);
//		msh2 = AddMesh(hpara,&off2);
//		SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
	off = (VZERO-comshift);

//	Make certain Soyuz Descent module is present
	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

/*
	if( status == 4 ) {
		static UINT groups0[5] = { 0, 1, 2, 3, 4 };
		
		animcomp1 = AddAnimationComponent( anim1, 0, 1,
			new MGROUP_ROTATE (
				msh2,
				groups0, 5,
				_V(0,0,1)-off2,
				_V(1,0,0),
				30*RAD
			)
		);

		animcomp2 = AddAnimationComponent( anim2, 0, 1,
			new MGROUP_ROTATE (
				msh2,
				groups0, 5,
				_V(0,0,1)-off2,
				_V(0,1,0),
				30*RAD
			)
		);

	}
*/
	status = 4;

}

void SoyuzDES::SetRetroStage() {

//	Sets parameters to base Descent Module.
//	ClearMeshes();	

	ClearAirfoilDefinitions ();
	SetPMI (ENTRY_PMI);
	SetCrossSections (ENTRY_CS);
	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

	SetRotDrag (ENTRY_RD);

	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);
	SetSurfaceFrictionCoeff(1, 1); // landing on heat shield

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	//	Fire Retro rockets
	SetThrusterLevel(th_srm,1.0);

	VECTOR3 off;
//	UINT msh, msh2;
//		msh2 = AddMesh(hpara,&off2);
//		SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
	off = (VZERO);

	AddDescVisual (off);
	if (shieldstatus == 0)
		AddHSVisual (off-_V(0,0,0.980));


	status = 5;

}
void SoyuzDES::SetGroundedStage() {

//	Sets parameters to base Descent Module.
//	ClearMeshes();	

	ClearAirfoilDefinitions ();
	SetPMI (ENTRY_PMI);
	SetCrossSections (ENTRY_CS);
	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

	SetRotDrag (ENTRY_RD);

	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);
	SetSurfaceFrictionCoeff(1, 1); // landing on heat shield

	//	We should be down so clear everything
//	ClearExhaustRefs();
//	ClearAttExhaustRefs();
//	ClearThrusterDefinitions();
//	DelPropellantResource(ph_second);
//	DelPropellantResource(ph_srm);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	VECTOR3 off;

	off = (VZERO);

	AddDescVisual (off);

	SetThrusterLevel(th_srm,0.0);

	status = 6;

}
void SoyuzDES::AddThrusters()
{

PARTICLESTREAMSPEC srb_contrail = {
	0, 12.0, 3, 150.0, 0.4, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
	PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
};
PARTICLESTREAMSPEC srb_exhaust = {
	0, 3.0, 40, 250.0, 0.04, 0.7, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
	PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
};

//	Main Engine definitions
	th_first = CreateThruster (_V(0,0,2.243)-comshift, _V(0,0,1),  100000.0, ph_first, 2992.0);
	SURFHANDLE tex = oapiRegisterExhaustTexture ("Exhaust2");
	srb_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");
	AddExhaust (th_first, 6.0, .42, _V(0,0.589,2.243), _V(0,.666,-1), tex);
	AddExhaust (th_first, 6.0, .42, _V(0,-0.589,2.243), _V(0,-.666,-1), tex);
	AddExhaust (th_first, 6.0, .42, _V(0.589,0,2.243), _V(.666,0,-1), tex);
	AddExhaust (th_first, 6.0, .42, _V(-0.589,0,2.243), _V(-.666,0,-1), tex);
	AddExhaustStream (th_first, _V(0,2.03,0.00), &srb_contrail);
	AddExhaustStream (th_first, _V(0,4.25,-3.75), &srb_exhaust);
	AddExhaustStream (th_first, _V(0,-2.03,0.00), &srb_contrail);
	AddExhaustStream (th_first, _V(0,-4.25,-3.75), &srb_exhaust);
	AddExhaustStream (th_first, _V(2.03,0,0.00), &srb_contrail);
	AddExhaustStream (th_first, _V(4.25,0,-3.75), &srb_exhaust);
	AddExhaustStream (th_first, _V(-2.03,0,0.00), &srb_contrail);
	AddExhaustStream (th_first, _V(-4.25,0,-3.75), &srb_exhaust);

//	Orbital RCS definitions
	th_pup[0] = CreateThruster (_V(-0.1, 1.38, -2.515)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[1] = CreateThruster (_V(0.1, 1.38, -2.515)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[2] = CreateThruster (_V(-1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[3] = CreateThruster (_V(1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_pup[0], &rcs_exhaust);
	AddExhaustStream(th_pup[1], &rcs_exhaust);
	AddExhaustStream(th_pup[2], &rcs_exhaust);
	AddExhaustStream(th_pup[3], &rcs_exhaust);

	th_pdown[0] = CreateThruster (_V(-0.1, -1.38, -2.515)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[1] = CreateThruster (_V(0.1, -1.38, -2.515)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[2] = CreateThruster (_V(-1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[3] = CreateThruster (_V(1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_pdown[0], &rcs_exhaust);
	AddExhaustStream(th_pdown[1], &rcs_exhaust);
	AddExhaustStream(th_pdown[2], &rcs_exhaust);
	AddExhaustStream(th_pdown[3], &rcs_exhaust);


	th_rleft[0] = th_pup[3];
	th_rleft[1] = th_pdown[2];

	th_rright[0] = th_pup[2];
	th_rright[1] = th_pdown[3];


	th_yleft[0] = CreateThruster (_V(-1.38, -0.1, -2.515)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[1] = CreateThruster (_V(-1.38, 0.1, -2.515)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[2] = CreateThruster (_V(1.2, -0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[3] = CreateThruster (_V(1.2, 0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_yleft[0], &rcs_exhaust);
	AddExhaustStream(th_yleft[1], &rcs_exhaust);
	AddExhaustStream(th_yleft[2], &rcs_exhaust);
	AddExhaustStream(th_yleft[3], &rcs_exhaust);
	

	th_yright[0] = CreateThruster (_V(1.38, -0.1, -2.515)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[1] = CreateThruster (_V(1.38,  0.1, -2.515)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[2] = CreateThruster (_V(-1.2, -0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[3] = CreateThruster (_V(-1.2,  0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_yright[0], &rcs_exhaust);
	AddExhaustStream(th_yright[1], &rcs_exhaust);
	AddExhaustStream(th_yright[2], &rcs_exhaust);
	AddExhaustStream(th_yright[3], &rcs_exhaust);

	CreateThrusterGroup( th_pup, 4, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 4, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_yleft, 4, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 4, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );
	
	CreateThrusterGroup( &th_first, 1, THGROUP_MAIN );

}

void SoyuzDES::AddDescVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_desc == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_desc = AddMesh (hdesc, &ofs);
		mesh_desc = InsertMesh (hentry, 1, &ofs);
		SetMeshVisibilityMode (mesh_desc, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void SoyuzDES::AddHSVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_HS == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_desc = AddMesh (hdesc, &ofs);
		mesh_HS = InsertMesh (hHSmesh, 4, &ofs);
		SetMeshVisibilityMode (mesh_HS, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void SoyuzDES::AddParaVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_para == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_para = AddMesh (hpara, &ofs);
		mesh_para = InsertMesh (hpara, 3, &ofs);
		SetMeshVisibilityMode (mesh_para, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void SoyuzDES::AddDrogueVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_drogue == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_drogue = AddMesh (hdrogue, &ofs);
		mesh_drogue = InsertMesh (hdrogue, 2, &ofs);
		SetMeshVisibilityMode (mesh_drogue, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void SoyuzDES::AddTMAVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_tma == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_tma = AddMesh (htma, &ofs);
		mesh_tma = InsertMesh (htma, 0, &ofs);
		SetMeshVisibilityMode (mesh_tma, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void SoyuzDES::ClearMeshes ()
{

//	Clearing meshes and mesh calls to orbiter
	VESSEL::ClearMeshes(true);
	mesh_tma = MESH_UNDEFINED;
	mesh_desc = MESH_UNDEFINED;
	mesh_HS = MESH_UNDEFINED;
	mesh_para = MESH_UNDEFINED;
	mesh_drogue = MESH_UNDEFINED;

}

void SoyuzDES::BlowOrbiter() {

//	JettisonComponent("SoyuzOrbitMod", "soyorbit", (VZERO-comshift)+_V(0.45,0.60,2.75) , _V(0,0,0.5), _V(0,0,0));

//	Separates the three secions of the Soyuz TMA
//	Orbital Module vessel definition
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = (VZERO-comshift)+_V(0.00,0.00,2.42);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (VZERO-comshift)+_V(0.00,0.00,2.42);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = -0.01;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-ER");
	oapiCreateVesselEx (name, "soyuzesc", &vs);

	start_time = oapiGetSimTime ();

	// remove TMA mesh
	DelMesh (mesh_tma);
	DelMesh (mesh_desc);
	DelMesh (mesh_HS);
//	oapiDeleteMesh (htma);
//	oapiDeleteMesh (hentry);
//	oapiDeleteMesh (hHSmesh);

	// reconfigure
	RecordEvent ("JET", "SM");

//	JettisonComponent("SoyuzServiceMod", "soyservice", (VZERO-comshift)+_V(0,0.2,-2.2) , _V(0,0,-0.5), _V(0,0,0));

	refresh = 1;
}

void SoyuzDES::BlowHeatShield() {

//	Separates the three secions of the Soyuz TMA
//	Orbital Module vessel definition
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = (VZERO-comshift)-_V(0.00,0.00,0.980);
//	VECTOR3 ofs = (VZERO-comshift)+_V(0.45,0.60,2.75);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (VZERO-comshift)-_V(0.00,0.00,0.980);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,-.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
//	vs.vrot.x = -0.01;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-HS");
	oapiCreateVesselEx (name, "SoyHeatShield", &vs);
	DelMesh (mesh_HS);
//	oapiDeleteMesh (hHSmesh);
	SetEmptyMass ( MASSENTRY - HS_EMPTY_MASS);
	shieldstatus = 1;

}
void SoyuzDES::CalcParaAngles() {

//	original code - untouched
	double aoa;
	double slip;

	aoa = GetAOA() - PI;
	if( aoa < -PI )
		aoa+=2.0*PI;
	
	slip = GetSlipAngle() - PI;
	if( slip < -PI )
		slip += 2.0*PI;

	if( aoa > 30*RAD )
		aoa = 30*RAD;
	else if( aoa < -30*RAD )
		aoa = -30*RAD;

	if( slip > 30*RAD )
		slip = 30*RAD;
	else if( slip < -30*RAD )
		slip = -30*RAD;
	
	aoa /= 30*RAD;
	slip /= 30*RAD;  // -1 to 1

	aoa /= 2.0;
	slip /= 2.0;	// -0.5 to 0.5

	aoa += 0.5;		// 0 to 1
	slip += 0.5;

	slip = 1.0-slip;   // correct for opposite sign
/*
	if( animcomp1 != NULL ) {
		SetAnimation( anim1, aoa );
	}
	if( animcomp2 != NULL ) {
		SetAnimation( anim2, slip );
	}
*/
}

void SoyuzDES::UpdateMesh ()
{
	// update animation states
	SetAnimation (anim_abort, abort_proc);

}

// --------------------------------------------------------------
// Define animation sequences for moving parts
// --------------------------------------------------------------
void SoyuzDES::DefineAnimations (void)
{
	// ***** 1. Solar Panel animation *****

	// 0.5 radians = 90 degrees - 0.0056 radians = 1 degree

	// 0.1111 radians = 20 degrees

	static UINT RightGrp[1] = {48};
	static MGROUP_ROTATE RightPanel (mesh_tma, RightGrp, 1,
		_V(-0.945,0.946,-5.565), _V(-.5,-.5,0), 90*RAD);
	static UINT FrontGrp[1] = {50};
	static MGROUP_ROTATE FrontPanel (mesh_tma, FrontGrp, 1,
		_V(-0.946,-0.947,-5.565), _V(-.5,.5,0), -90*RAD);
	static UINT LeftGrp[1] = {49};
	static MGROUP_ROTATE LeftPanel (mesh_tma, LeftGrp, 1,
		_V(0.944,-0.952,-5.565), _V(.5,.5,0), 90*RAD);
	static UINT RearGrp[1] = {47};
	static MGROUP_ROTATE RearPanel (mesh_tma, RearGrp, 1,
		_V(0.946,0.946,-5.565), _V(.5,-.5,0), -90*RAD);

	anim_abort = CreateAnimation (i_state);

	AddAnimationComponent (anim_abort, 0.0, 1.00, &RightPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &LeftPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &FrontPanel);
	AddAnimationComponent (anim_abort, 0.0, 1.00, &RearPanel);

}

void SoyuzDES::OperateAbort (DoorStatus action)
{

	abort_status = action;
	RecordEvent ("ABORT", action == DOOR_CLOSING ? "Inactive" : "Active");
}

void SoyuzDES::RevertAbort ()
{

	OperateAbort (abort_status == DOOR_CLOSED || abort_status == DOOR_CLOSING ?
		DOOR_OPENING : DOOR_CLOSING);
}

// ==============================================================
// Overloaded callback functions
// ==============================================================

void SoyuzDES::clbkSetClassCaps (FILEHANDLE cfg)
{

	if (!oapiReadItem_bool (cfg, "RenderCockpit", render_cockpit))
		render_cockpit = false;

}

void SoyuzDES::clbkSetStateEx (const void *status)
{
	// default parameter initialisation
	DefSetStateEx (status);

	// reset vessel-specific parameters to defaults
	SetMainStage();
	refresh = 1;

}

void SoyuzDES::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{

    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		if( !strnicmp( line, "CONFIGURATION", 13 ) ) {
            sscanf (line+13, "%d", &status);
		} else if (!strnicmp (line, "ABORT", 5)) {
			sscanf (line+5, "%d%lf", &abort_status, &abort_proc);
		} else { 
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
	}


	ClearMeshes();
	/* always set the stage whenever something may have changed */
	switch (status) {
	case 0:		
	case 1:		// Soyuz TMA in orbit
		SetMainStage();
		UpdateMesh();
		break;
	case 2:		// Soyuz Descent Module
		SetEntryStage();
		break;
	case 3:		// Drogue deployment
		SetDrogueStage();
		break;
	case 4:		// Main chute deployment
		SetParaStage();
		break;
	case 5:		//Retro firing at approach to ground
		SetRetroStage();
		break;
	case 6:		// Touch down and sit.
		SetGroundedStage();
	}
	refresh = 1;

}

void SoyuzDES::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// custom parameters
	oapiWriteScenario_int (scn, "CONFIGURATION", status);

	sprintf (cbuf, "%d %0.4f", abort_status, abort_proc);
	oapiWriteScenario_string (scn, "ABORT", cbuf);

//	SaveDefaultStateEx(scn);

}

void SoyuzDES::clbkFocusChanged (bool getfocus, OBJHANDLE newv, OBJHANDLE oldv)
{
	if (getfocus) {
		oapiDisableMFDMode (MFD_LANDING);
		// no VTOL MFD mode for Atlantis
	}
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}

void SoyuzDES::clbkPostStep (double simt, double simdt, double mjd)
{


	switch (status) {
	case 0:		// Soyuz TMA in orbit
			SetMainStage();
			UpdateMesh();
			break;
	case 1:		// Soyuz Descent Module
		if ( GetPropellantMass (ph_first) < 2.0 ) {
			SetThrusterLevel(th_first,0.0);
			if (!bManualSeparate) {
				if (start_alt > GetAltitude() ) {
					RevertAbort();
					bManualSeparate = true;
				}

				else {
					start_alt = GetAltitude ();
				}
			}
		}
		else
			SetThrusterLevel(th_first,1.0);
//		if( bManualSeparate && abort_status == DOOR_CLOSED) {
		if( abort_status == DOOR_CLOSED) {
			BlowOrbiter();
			ClearMeshes();
			SetEntryStage();
			bManualSeparate = false;
		}
			break;
	case 2:		// Drogue deployment
		if ((oapiGetSimTime () - start_time) > chute_timer) {
			if( GetAltitude() < 12000 && GetAirspeed() < 300 ) {
				ClearMeshes();
				SetDrogueStage();
			}
		}
			break;
	case 3:		// Main chute deployment
		CalcParaAngles();
		if ( GetAltitude() < 2003.0 && shieldstatus == 0) {
			BlowHeatShield();
		}
		if ( GetAltitude() < 2000.0 && GetAirspeed() < 80.0 ) {
			oapiDeleteMesh (hdrogue);
			ClearMeshes();
			SetParaStage();
		}
			break;
	case 4:		//Retro firing at approach to ground
		CalcParaAngles();
		if( GetAltitude() < 2.0) {
			oapiDeleteMesh (hpara);
			ClearMeshes();
			SetRetroStage();
			SetThrusterLevel(th_srm,1.0);
		}
			break;
	case 5:		// Touch down and sit.
		SetThrusterLevel(th_srm,1.0);
		if( GroundContact()) {
			SetThrusterLevel(th_srm,0.0);
			ClearMeshes();
			SetGroundedStage();
		}
	}

	if (abort_status >= DOOR_CLOSING) {
		double da = simdt * ABORT_OPERATING_SPEED;
		if (abort_status == DOOR_CLOSING) { // retract gear
			if (abort_proc > 0.5) abort_proc = max (0.0, abort_proc-da);
			else  {
				abort_status = DOOR_CLOSED;
			}
		} else {                           // deploy gear
			if (abort_proc < 1.0) {
				abort_proc = min (1.0, abort_proc+da);  //Door Opening
			}
			else  {
				abort_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_abort, abort_proc);
	}

}

bool SoyuzDES::clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event)
{

	return false;

}

void SoyuzDES::clbkVisualCreated (VISHANDLE _vis, int refcount)
{

	if (refcount > 1) return; // we don't support more than one visual per object
	vis = _vis;

	//UpdateMesh ();
}

void SoyuzDES::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{

	if (vis == _vis) vis = NULL;

}

void SoyuzDES::clbkAnimate (double simt)
{

//	UpdateMesh ();

}

bool SoyuzDES::clbkLoadVC (int id)
{

	switch (id) {
	case 0:
		oapiVCRegisterHUD (&huds);
		oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
		oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
		return true;
	default:
		return false;
	}

}

/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */
int SoyuzDES::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{

	if (!down) return 0; // only process keydown events


		switch (key) {
		case OAPI_KEY_J:  // "Jettison"
			if (!Playback()) {
				bManualSeparate = true;
			}
			return 1;
		case OAPI_KEY_K:  // "Solar Panels"
//			if (!Playback()) RevertAbort ();
			return 1;
		}


//			VESSELSTATUS vs;
//	GetStatus(vs);
	
//	if (KEYMOD_SHIFT (kstate)) {
		// reserved for MFDs
//	}
//	else if (KEYMOD_CONTROL (kstate)) 
//	{
//		if (KEYDOWN (kstate, OAPI_KEY_J)  ) 
//		{ 
//			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				if( status == 0 ) {
//					bManualSeparate = true;
//				}
//				return 1;
//			}
//		}
//	}
//	else 
//	{ // unmodified keys
//		if (KEYDOWN (kstate, OAPI_KEY_K)) { // "Jettison Stage"
//			if(oapiAcceptDelayedKey (OAPI_KEY_K, 0.5)) {
//				if( animlevel > 0.5 )
//					animactive = 2;
//				else
//					animactive = 1;
//				return 1;
//			}
//		}
//		if (KEYDOWN (kstate, OAPI_KEY_J) && !KEYMOD_CONTROL(kstate) && !KEYMOD_SHIFT(kstate) ) 
//		{ 
//			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				DeployPayload();
//				refresh=1;
//				return 1;
//			}
//		}
//	}

	return 0;

}
// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Module initialisation
// --------------------------------------------------------------
DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SoyuzDES (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SoyuzDES*)vessel;
}
