// ==============================================================
//                 SERVICEER MODULE: Orion
//                  Part of the SERVICEER SDK
//          Copyright (C) 2001-2003 Martin Schweiger
//                   All rights reserved
//
// SoyHeatShield.cpp
// Reference implementation of Atlantis Tank vessel class module
// Note: This module takes control of the tank after separation
// from the orbiter.
// ==============================================================

#define ORBITER_MODULE
#include "soyuztma.h"
#include "math.h"

// ==============================================================
// Specialised vessel class CEVIS
// ==============================================================

// Constructor
SoyHeatShield::SoyHeatShield (OBJHANDLE hObj)
: VESSEL2(hObj)
{
	// preload mesh
	hHSmesh = oapiLoadMeshGlobal ("ACRMHS");
}

// ==============================================================
// Callback functions
// ==============================================================

// Set Tank class specs
void SoyHeatShield::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetEnableFocus (false);
	// Tank cannot receive input focus

	SetSize (4);
	SetEmptyMass (HS_EMPTY_MASS);

	SetMaxFuelMass (HS_MAX_PROPELLANT_MASS);
	// Note that the Tank instance is only created after separation from
	// the orbiter, so the actual fuel mass will always be much smaller

	SetISP (HS_ISP0);

	SetMaxThrust (ENGINE_MAIN, 0);
	SetMaxThrust (ENGINE_RETRO, 0);
	SetMaxThrust (ENGINE_HOVER, 0);
	SetMaxThrust (ENGINE_ATTITUDE, 0);
	// Tank has no engines of its own

	SetCW (HS_CW[0], HS_CW[1], HS_CW[2], HS_CW[3]);
	SetCrossSections (HS_CS);
	SetRotDrag (HS_RD);
	SetPMI (HS_PMI);
	SetPitchMomentScale (1e-4);
	SetBankMomentScale (1e-4);

	VECTOR3 co = {0,0,0};
	SetCameraOffset (co);
	// Note that the camera offset should not be required
	// since the Tank doesn't define a 'cockpit'

	SetCOG_elev (0.0);
	SetTouchdownPoints (_V(-2.70, -2.70, -0.173), _V(2.70, 0.0, -0.173), _V(2.70, -2.70, -0.173));
	SetLiftCoeffFunc (0);

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	mesh_HS = AddMesh (hHSmesh);
	SetMeshVisibilityMode (mesh_HS, MESHVIS_ALWAYS);

}

// Simulation time step
void SoyHeatShield::clbkPostStep (double simt, double simdt, double mjd)
{
//	if (GetAltitude() < 35.0) oapiDeleteVessel (GetHandle());
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SoyHeatShield (hvessel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SoyHeatShield*)vessel;
}

// Vessel status initialisation
DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}
