// ==============================================================
// $Id: soyuztma.cpp,v 1.11 2006/03/18 21:06:20 dh219 Exp $
// Soyuz TMA by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <stdio.h>
#include <fstream>
#include "soyuztma.h"

PARTICLESTREAMSPEC rcs_exhaust = {
0, 0.05, 3000, 5, 0.1, 0.1, 1, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust = {
0, 0.4, 3000, 150, 0.05, 0.5, 5, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};
PARTICLESTREAMSPEC oms_exhaust2 = {
0, 0.4, 3000, 50, 0.05, 0.1, 5.0/3.0, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
PARTICLESTREAMSPEC::LVL_PLIN, 0, 2,
PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};

void CoeffFunc (double aoa, double M, double Re, double *cl, double *cm, double *cd)

{
	const int nlift = 11;
	double factor,dfact,lfact,frac,drag,lift;
	static const double AOA[nlift] =
		{-180.*RAD,-160.*RAD,-150.*RAD,-120.*RAD,-90.*RAD,0*RAD,90.*RAD,120.*RAD,150.*RAD,160.*RAD,180.*RAD};
	static const double Mach[17] = {0.0,0.7,0.9,1.1,1.2,1.35,1.65,2.0,3.0,5.0,8.0,10.5,13.5,18.2,21.5,31.0,50.0};
	static const double LFactor[17] = {0.3,0.392,0.466,0.607,0.641,0.488,0.446,0.435,0.416,0.415,0.405,0.400,0.385,0.385,0.375,0.35,0.33};
	static const double DFactor[17] = {0.9,0.944,0.991,1.068,1.044,1.270,1.28,1.267,1.213,1.134,1.15,1.158,1.8,1.8,1.193,1.224,1.25};
	static const double CL[nlift]  = {0.0,-0.9,-1.1,-0.5,0.0,0.0,0.0,0.5,1.1,0.9,0.0};
	static const double CM[nlift]  = {0.0,0.004,0.006,0.012,0.015,0.0,-0.015,-0.012,-0.006,-0.004,0.};
	static const double CD[nlift]  = {1.143,1.0,1.0,0.8,0.8,0.8,0.8,0.8,1.0,1.0,1.143};
	int j;
	factor = 0.0;
	dfact = 0.9;
	lfact = 1.0;
	for(j = 0; (j < 16) && (Mach[j+1] < M); j++);
	frac = (M-Mach[j])/(Mach[j+1]-Mach[j]);
	drag = dfact*(frac*DFactor[j+1]+(1.0-frac)*DFactor[j]);
	lift = drag * lfact*(frac*LFactor[j+1]+(1.0-frac)*LFactor[j]);
	for(j = 0; (j < nlift-1) && (AOA[j+1] < aoa); j++);
	frac = (aoa-AOA[j])/(AOA[j+1]-AOA[j]);
	*cd = drag*(frac*CD[j+1]+(1.0-frac)*CD[j]);
	*cl = lift*(frac*CL[j+1]+(1.0-frac)*CL[j]);
	*cm = factor*(frac*CM[j+1]+(1.0-frac)*CM[j]);
}

struct {
	HINSTANCE hDLL;
} g_Param;

TMA::TMA (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj)
{
	ph_first = NULL;
	ph_second = NULL;
	ph_srm = NULL;

	htma = oapiLoadMesh("soyuz");
	hentry = oapiLoadMesh("soyuz-descent");
	hpara = oapiLoadMesh("soychute");
	hdrogue = oapiLoadMesh("soydrogue");
	hHSmesh = oapiLoadMesh("ACRMHS");

	mesh_tma             = MESH_UNDEFINED;
	mesh_para            = MESH_UNDEFINED;
	mesh_drogue          = MESH_UNDEFINED;
	mesh_desc            = MESH_UNDEFINED;
	mesh_HS              = MESH_UNDEFINED;

	VZERO = _V(0,0,0);

	status = 0;
	shieldstatus = 0;
	targetstatus = 0;

	comshift = _V(0,0,0);

//	anim1 = CreateAnimation (0.5);
//	anim2 = CreateAnimation (0.5);
	anim3 = CreateAnimation(0);

	animcomp1 = NULL;
	animcomp2 = NULL;

	int i;
	for( i = 0 ; i < 20 ; i++ ) {
		animcomp3[i] = NULL;
	}

	xproc = 0.0;

	chute = NULL;

	reenteractive = false;
	reentercoasting = false;
	target = 6445000.0;
	bManualSeparate = false;

	animstate = 0;
	animlevel = 0.0;
	refresh = 0;
	animactive = 0;

	dh=NULL;

}




TMA::~TMA()
{
	delete th_first;
}
//void TMA::TimeStep(  double simt )
//{
//	/* CVE-lite function ! */
//	/* IMPORTANT */
//	/* This function loops through the pending GPDS requests and returns a bool (true/false)
//	/* if any GPDS requests have been implemented.
//	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
//	/* call to the (repeatable) SetMainStage(); */

//	if( CheckCentralMessage() ) {
//		while( CheckCentralMessage() ) ;
//		SetMainStage();
//	}



//}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void TMA::SetMainStage( )
{

	SetAnimation( anim3, 0.0 );

	VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	
	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshes();

	SetSize (15);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
//	SetEmptyMass ( MASS + CalculateMass() );
	SetEmptyMass ( MASS );

	ShiftCG(VZERO-comshift);
	comshift = _V(0,0,0);
	ShiftCG(comshift);


	SetTouchdownPoints (_V(0,-2,4)-comshift,_V(2,-2,-4)-comshift,_V(-2,-2,-4)-comshift);

	SetPMI (_V(5,5,0.85));
	SetCrossSections (_V(15,32,6));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.1,0.1,0.01));
	SetPitchMomentScale (-3e-6);
	SetBankMomentScale (3e-6);
	SetLiftCoeffFunc(0); 

	SetSurfaceFrictionCoeff (0.05, 0.5);
	SetCameraOffset( _V( 0.0, -1.50, 1.5)-comshift );


	VECTOR_ZERO = (comshift*-1.0);

	/* we're not using COMMON_MESH today, so draw our own */
	AddTMAVisual (VECTOR_ZERO);
//	UINT mshid = AddMesh("soyuz", &VECTOR_ZERO );
//	SetMeshVisibilityMode( mshid, MESHVIS_ALWAYS );

	Animations(mesh_tma);

//	MeshVisibilityPreference(MESHVIS_ALWAYS);


	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */
	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
//	Draw(comshift*-1.0);

	
	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if( ph_first == NULL ) {
		ph_first = CreatePropellantResource(FUEL);
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}

	if( !dh )
		dh = CreateDock(_V(0,0,4.615),_V(0,0,1),_V(0,1,0));

	SetDefaultPropellantResource (ph_first); 


	AddThrusters();
	EnableTransponder(true);
	status = 1;
}


void TMA::SetEntryStage() {


	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,-0.5);
	ShiftCG(comshift);
	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );

	VECTOR3 off;
//	UINT msh;
	off = (VZERO-comshift);

	int i;
	for( i = 0 ; i < 20 ; i++ ) {
		if( animcomp3[i] != NULL ) {
			DelAnimationComponent(anim3, animcomp3[i] );
			animcomp3[i] = NULL;
		}
	}
/*
	if( animcomp1 != NULL )
		DelAnimationComponent(anim1, animcomp1 );
	if( animcomp2 != NULL )
		DelAnimationComponent(anim2, animcomp2 );
*/	

	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
	ClearMeshes();	

	SetSize (5);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

	SetPMI (ENTRY_PMI);
	SetCrossSections (REENTRY_CS);
	SetCW (REENTRY_CW[0],REENTRY_CW[1],REENTRY_CW[2],REENTRY_CW[3]);
	CreateAirfoil(LIFT_VERTICAL, _V(0.0, 0.05, 0.94), CoeffFunc, 2.24, 3.94, 1.0);

	SetRotDrag (REENTRY_RD);
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}
	
	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}

	if( ph_second && GetPropellantMass(ph_second) > FUELRCS ) {
		SetPropellantMass(ph_second, FUELRCS);
	}

	if (GetPropellantMass(ph_second) < FUELRCS) {
		SetPropellantMass(ph_second, FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

	status = 2;

}

void TMA::SetDrogueStage() {


	int i;
	for( i = 0 ; i < 20 ; i++ ) {
		if( animcomp3[i] != NULL ) {
			DelAnimationComponent(anim3, animcomp3[i] );
			animcomp3[i] = NULL;
		}
	}
	
	
	ClearAirfoilDefinitions ();
	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,0);
	ShiftCG(comshift);

//	SetPMI (ENTRY_PMI);
//	SetCrossSections (ENTRY_CS);
//	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

//	SetRotDrag (ENTRY_RD);
	SetPMI (_V(2,2,0.55));
	SetCrossSections (_V(5,5,40));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.05,0.05,0.05));
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);

	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );

	
	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
	ClearMeshes();	

	SetSize (80);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

	VECTOR3 off, off2;
//	UINT msh, msh2;
	off = (VZERO-comshift);
	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
	off2 = (VZERO-comshift)+_V(0,0,72);
	AddDrogueVisual (off2);
//	msh2 = AddMesh(hdrogue,&off2);
//	SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

/*
	if( animcomp1 != NULL )
		DelAnimationComponent(anim1, animcomp1 );
	if( animcomp2 != NULL )
		DelAnimationComponent(anim2, animcomp2 );
*/

/*	static UINT groups0[5] = { 0, 1, 2, 3, 4 };
	
	animcomp1 = AddAnimationComponent( anim1, 0, 1,
		new MGROUP_ROTATE (
			msh2,
			groups0, 5,
			_V(0,0,1)-off2,
			_V(1,0,0),
			30*RAD
		)
	);

	animcomp2 = AddAnimationComponent( anim2, 0, 1,
		new MGROUP_ROTATE (
			msh2,
			groups0, 5,
			_V(0,0,1)-off2,
			_V(0,1,0),
			30*RAD
		)
	);
*/
	status = 3;

}

void TMA::SetParaStage() {
	

//	Sets up Main parachute deployment

//	removes animation components
//	technique used in earlier version of orbiter but still supported
	int i;
	for( i = 0 ; i < 20 ; i++ ) {
		if( animcomp3[i] != NULL ) {
			DelAnimationComponent(anim3, animcomp3[i] );
			animcomp3[i] = NULL;
		}
	}
	
/*	if( animcomp1 != NULL )
		DelAnimationComponent(anim1, animcomp1 );
	if( animcomp2 != NULL )
		DelAnimationComponent(anim2, animcomp2 );
*/
	
//	Removes arfoil definitions to allow SetCrossSections to work as Main Chute.
//	Another method to use is a different airfoil definition
	ClearAirfoilDefinitions ();

	ShiftCG(VZERO-comshift);
	comshift = _V(0,-0.1,0);
	ShiftCG(comshift);

//	SetPMI (ENTRY_PMI);
//	SetCrossSections (ENTRY_CS);
//	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

//	SetRotDrag (ENTRY_RD);

//	Sets parameters to emulate a main chute.
	SetPMI (_V(2,2,0.55));
	SetCrossSections (_V(5,5,1000));
	SetCW (1.1, 2.2, 3.5, 3.5);

	SetRotDrag (_V(0.05,0.05,0.05));
	SetPitchMomentScale (-1e-6);
	SetBankMomentScale (-1e-6);
	SetLiftCoeffFunc(0);


//	comshift = _V(0,-0.18,-1.265);

	SetCameraOffset( _V( 0.0, -1.65, -0.1)-comshift );
	
	ClearExhaustRefs();
	ClearAttExhaustRefs();
  	ClearThrusterDefinitions();
	ClearMeshes();	

	SetSize (80);
//	SetEmptyMass ( MASSENTRY + CalculateMass() );
	SetEmptyMass ( MASSENTRY );
	
	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);

//	If low enough in the atmosphere, Main engine and RCS not needed
//	better to deconfigure Thruster Definitions only, but
//	have not tested what additional propellant weight will do to this section.
	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	VECTOR3 off, off2;
//	UINT msh, msh2;
	off2 = (VZERO-comshift)+_V(0,0,72);
//	Add Parachute mesh
	AddParaVisual (off2);
//		msh2 = AddMesh(hpara,&off2);
//		SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
	off = (VZERO-comshift);

//	Make certain Soyuz Descent module is present
	AddDescVisual (off);
	AddHSVisual (off-_V(0,0,0.980));
//	msh = AddMesh(hentry,&off);
//	SetMeshVisibilityMode(msh,MESHVIS_ALWAYS);
//	Draw(VZERO-comshift);

/*
	if( status == 4 ) {
		static UINT groups0[5] = { 0, 1, 2, 3, 4 };
		
		animcomp1 = AddAnimationComponent( anim1, 0, 1,
			new MGROUP_ROTATE (
				msh2,
				groups0, 5,
				_V(0,0,1)-off2,
				_V(1,0,0),
				30*RAD
			)
		);

		animcomp2 = AddAnimationComponent( anim2, 0, 1,
			new MGROUP_ROTATE (
				msh2,
				groups0, 5,
				_V(0,0,1)-off2,
				_V(0,1,0),
				30*RAD
			)
		);

	}
*/
	status = 4;

}

void TMA::SetRetroStage() {

//	Sets parameters to base Descent Module.
	ClearMeshes();	

	ClearAirfoilDefinitions ();
	SetPMI (ENTRY_PMI);
	SetCrossSections (ENTRY_CS);
	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

	SetRotDrag (ENTRY_RD);

	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);
	SetSurfaceFrictionCoeff(1, 1); // landing on heat shield

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	//	Fire Retro rockets
	SetThrusterLevel(th_srm,1.0);

	VECTOR3 off;
//	UINT msh, msh2;
//		msh2 = AddMesh(hpara,&off2);
//		SetMeshVisibilityMode(msh2,MESHVIS_ALWAYS);
	off = (VZERO);

	AddDescVisual (off);
	if (shieldstatus == 0)
		AddHSVisual (off-_V(0,0,0.980));


	status = 5;

}
void TMA::SetGroundedStage() {

//	Sets parameters to base Descent Module.
	ClearMeshes();	

	ClearAirfoilDefinitions ();
	SetPMI (ENTRY_PMI);
	SetCrossSections (ENTRY_CS);
	SetCW (ENTRY_CW[0],ENTRY_CW[1],ENTRY_CW[2],ENTRY_CW[3]);

	SetRotDrag (ENTRY_RD);

	SetTouchdownPoints (_V(-1,-1,-1.14)-comshift,_V(0,1,-1.14)-comshift,_V(1,-1,-1.14)-comshift);
	SetSurfaceFrictionCoeff(1, 1); // landing on heat shield

	//	We should be down so clear everything
//	ClearExhaustRefs();
//	ClearAttExhaustRefs();
//	ClearThrusterDefinitions();
//	DelPropellantResource(ph_second);
//	DelPropellantResource(ph_srm);

	if( ph_first ) {
		DelPropellantResource(ph_first);
		ph_first = NULL;
	}

	if( ph_second == NULL ) {
		ph_second = CreatePropellantResource(FUELRCS);
	}

	SetDefaultPropellantResource (ph_second); 

	th_yleft[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_yright[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(1,0,0), 50.0, ph_second, 300.0,300.0);

	th_pup[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,1,0), 50.0, ph_second, 300, 300.0);
	th_pdown[0] = CreateThruster (_V(0, 0, 1)-comshift, _V(0,-1,0), 50.0, ph_second, 300.0, 300.0);

	th_rleft[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(-1,0,0), 50.0, ph_second, 300.0, 300.0);
	th_rright[0] = CreateThruster (_V(0, 1, 0)-comshift, _V(1,0,0), 50.0, ph_second, 300.0, 300.0);

	CreateThrusterGroup( th_yleft, 1, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 1, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_pup, 1, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 1, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_rleft, 1, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 1, THGROUP_ATT_BANKRIGHT );

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);

//	Making certain Retro rockets are ready
	if( ph_srm == NULL ) {
		ph_srm = CreatePropellantResource(FUELSRM);
	}
	
	th_srm = CreateThruster (_V(0, 0, -1.14), _V(0,0,1), 60000.0, ph_srm, 200.0, 200.0);
	AddExhaustStream(th_srm, _V(0,1,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(-0.7,-0.7,-1.14), &oms_exhaust);
	AddExhaustStream(th_srm, _V(0.7,-0.7,-1.14), &oms_exhaust);

	VECTOR3 off;

	off = (VZERO);

	AddDescVisual (off);

	SetThrusterLevel(th_srm,0.0);

	status = 6;

}
void TMA::AddThrusters()
{

//	Main Engine definitions
	th_first = CreateThruster (_V(0,0,-2.64)-comshift, _V(0,0,1),  3924.0, ph_first, 2992.0);
	AddExhaustStream(th_first, &oms_exhaust);
	AddExhaustStream(th_first, &oms_exhaust2);

//	Orbital RCS definitions
	th_pup[0] = CreateThruster (_V(-0.1, 1.38, -2.515)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[1] = CreateThruster (_V(0.1, 1.38, -2.515)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[2] = CreateThruster (_V(-1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pup[3] = CreateThruster (_V(1.12, -0.25, 0)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_pup[0], &rcs_exhaust);
	AddExhaustStream(th_pup[1], &rcs_exhaust);
	AddExhaustStream(th_pup[2], &rcs_exhaust);
	AddExhaustStream(th_pup[3], &rcs_exhaust);

	th_pdown[0] = CreateThruster (_V(-0.1, -1.38, -2.515)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[1] = CreateThruster (_V(0.1, -1.38, -2.515)-comshift, _V(0,1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[2] = CreateThruster (_V(-1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);
	th_pdown[3] = CreateThruster (_V(1.12, 0.25, 0)-comshift, _V(0,-1,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_pdown[0], &rcs_exhaust);
	AddExhaustStream(th_pdown[1], &rcs_exhaust);
	AddExhaustStream(th_pdown[2], &rcs_exhaust);
	AddExhaustStream(th_pdown[3], &rcs_exhaust);


	th_rleft[0] = th_pup[3];
	th_rleft[1] = th_pdown[2];

	th_rright[0] = th_pup[2];
	th_rright[1] = th_pdown[3];


	th_yleft[0] = CreateThruster (_V(-1.38, -0.1, -2.515)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[1] = CreateThruster (_V(-1.38, 0.1, -2.515)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[2] = CreateThruster (_V(1.2, -0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yleft[3] = CreateThruster (_V(1.2, 0.1, 0)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_yleft[0], &rcs_exhaust);
	AddExhaustStream(th_yleft[1], &rcs_exhaust);
	AddExhaustStream(th_yleft[2], &rcs_exhaust);
	AddExhaustStream(th_yleft[3], &rcs_exhaust);
	

	th_yright[0] = CreateThruster (_V(1.38, -0.1, -2.515)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[1] = CreateThruster (_V(1.38,  0.1, -2.515)-comshift, _V(-1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[2] = CreateThruster (_V(-1.2, -0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);
	th_yright[3] = CreateThruster (_V(-1.2,  0.1, 0)-comshift, _V(1,0,0), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_yright[0], &rcs_exhaust);
	AddExhaustStream(th_yright[1], &rcs_exhaust);
	AddExhaustStream(th_yright[2], &rcs_exhaust);
	AddExhaustStream(th_yright[3], &rcs_exhaust);

	th_left[0] = th_yleft[2];
	th_left[1] = th_yleft[3];

	th_right[0] = th_yright[2];
	th_right[1] = th_yright[3];
	
	th_up[0] = th_pup[2];
	th_up[1] = th_pup[3];

	th_down[0] = th_pdown[2];
	th_down[1] = th_pdown[3];




	th_fore[0] = CreateThruster (_V(-1.1, 0, -2.64)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[1] = CreateThruster (_V(1.1, 0, -2.64)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[2] = CreateThruster (_V(0, 1.1, -2.64)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);
	th_fore[3] = CreateThruster (_V(0, -1.1, -2.64)-comshift, _V(0,0,1), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_fore[0], &rcs_exhaust);
	AddExhaustStream(th_fore[1], &rcs_exhaust);
	AddExhaustStream(th_fore[2], &rcs_exhaust);
	AddExhaustStream(th_fore[3], &rcs_exhaust);


	th_aft[0] = CreateThruster (_V(-1.15, 0, 0.15)-comshift, _V(0.141,0,-0.99), 140.0, ph_first, 2903.0, 0.0);
	th_aft[1] = CreateThruster (_V(1.15, 0, 0.15)-comshift, _V(-0.141,0,-0.99), 140.0, ph_first, 2903.0, 0.0);

	AddExhaustStream(th_aft[0], &rcs_exhaust);
	AddExhaustStream(th_aft[1], &rcs_exhaust);


	CreateThrusterGroup( th_pup, 4, THGROUP_ATT_PITCHUP );
	CreateThrusterGroup( th_pdown, 4, THGROUP_ATT_PITCHDOWN );
	CreateThrusterGroup( th_yleft, 4, THGROUP_ATT_YAWLEFT );
	CreateThrusterGroup( th_yright, 4, THGROUP_ATT_YAWRIGHT );
	CreateThrusterGroup( th_rleft, 2, THGROUP_ATT_BANKLEFT );
	CreateThrusterGroup( th_rright, 2, THGROUP_ATT_BANKRIGHT );
	
	CreateThrusterGroup( th_left, 2, THGROUP_ATT_LEFT );
	CreateThrusterGroup( th_right, 2, THGROUP_ATT_RIGHT );
	CreateThrusterGroup( th_down, 2, THGROUP_ATT_DOWN );
	CreateThrusterGroup( th_up, 2, THGROUP_ATT_UP );
	CreateThrusterGroup( th_fore, 4, THGROUP_ATT_FORWARD );
	CreateThrusterGroup( th_aft, 2, THGROUP_ATT_BACK );

	CreateThrusterGroup( &th_first, 1, THGROUP_MAIN );
	CreateThrusterGroup( th_aft, 2, THGROUP_RETRO );



}

void TMA::AddDescVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_desc == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_desc = AddMesh (hdesc, &ofs);
		mesh_desc = InsertMesh (hentry, 1, &ofs);
		SetMeshVisibilityMode (mesh_desc, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void TMA::AddHSVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_HS == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_desc = AddMesh (hdesc, &ofs);
		mesh_HS = InsertMesh (hHSmesh, 4, &ofs);
		SetMeshVisibilityMode (mesh_HS, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void TMA::AddParaVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_para == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_para = AddMesh (hpara, &ofs);
		mesh_para = InsertMesh (hpara, 3, &ofs);
		SetMeshVisibilityMode (mesh_para, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void TMA::AddDrogueVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_drogue == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_drogue = AddMesh (hdrogue, &ofs);
		mesh_drogue = InsertMesh (hdrogue, 2, &ofs);
		SetMeshVisibilityMode (mesh_drogue, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void TMA::AddTMAVisual (const VECTOR3 &ofs)
{

//	method to insert mesh and manually index them
//	nice for when you have several attached modules that can be jettisoned
	if (mesh_tma == MESH_UNDEFINED) {

		// ***** Load mesh
//		mesh_tma = AddMesh (htma, &ofs);
		mesh_tma = InsertMesh (htma, 0, &ofs);
		SetMeshVisibilityMode (mesh_tma, MESHVIS_EXTERNAL | MESHVIS_VC);
	}
}

void TMA::ClearMeshes ()
{

//	Clearing meshes and mesh calls to orbiter
	VESSEL::ClearMeshes();
	mesh_tma = MESH_UNDEFINED;
	mesh_desc = MESH_UNDEFINED;
	mesh_para = MESH_UNDEFINED;
	mesh_drogue = MESH_UNDEFINED;

}

void TMA::BlowOrbiter() {

//	JettisonComponent("SoyuzOrbitMod", "soyorbit", (VZERO-comshift)+_V(0.45,0.60,2.75) , _V(0,0,0.5), _V(0,0,0));

//	Separates the three secions of the Soyuz TMA
//	Orbital Module vessel definition
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = (VZERO-comshift)+_V(0.00,0.00,2.42);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (VZERO-comshift)+_V(0.00,0.00,2.42);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = -0.01;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-Orbit Mod");
	oapiCreateVesselEx (name, "soyorbit", &vs);

//	VESSELSTATUS2 vs;
//	Service Module vessel definition
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	ofs = (VZERO-comshift)+_V(0,0.2,-2.2);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (VZERO-comshift)+_V(0,0.2,-2.2);
	VECTOR3 rofs2, rvel2 = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel2 = {0,0,-.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel2, rofs2);
	vs.rvel.x = rvel2.x+rofs2.x;
	vs.rvel.y = rvel2.y+rofs2.y;
	vs.rvel.z = rvel2.z+rofs2.z;
	vs.vrot.x = -0.01;
	vs.status = 0;
	name[256];
	strcpy (name, GetName());  strcat (name, "-Service Mod");
	oapiCreateVesselEx (name, "soyservice", &vs);

	// remove TMA mesh
//	DelMesh (mesh_TMA);
	oapiDeleteMesh (htma);

	// reconfigure
	RecordEvent ("JET", "SM");

//	JettisonComponent("SoyuzServiceMod", "soyservice", (VZERO-comshift)+_V(0,0.2,-2.2) , _V(0,0,-0.5), _V(0,0,0));

	status = 1;
	refresh = 1;
}

void TMA::BlowHeatShield() {

//	Separates the three secions of the Soyuz TMA
//	Orbital Module vessel definition
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof (vs));
	vs.version = 2;
	GetStatusEx (&vs);
	vs.flag = VS_FUELRESET | VS_THRUSTRESET;
	VECTOR3 ofs = (VZERO-comshift)-_V(0.00,0.00,0.980);
//	VECTOR3 ofs = (VZERO-comshift)+_V(0.45,0.60,2.75);
	if (Playback()) // necessary because during playback the CG shift occurs before separation
		ofs -= (VZERO-comshift)-_V(0.00,0.00,0.980);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,-.5};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
//	vs.vrot.x = -0.01;
	vs.status = 0;
	char name[256];
	strcpy (name, GetName());  strcat (name, "-HS");
	oapiCreateVesselEx (name, "SoyHeatShield", &vs);
	oapiDeleteMesh (hHSmesh);
	SetEmptyMass ( MASSENTRY - HS_EMPTY_MASS);
	shieldstatus = 1;

}
void TMA::CalcParaAngles() {

//	original code - untouched
	double aoa;
	double slip;

	aoa = GetAOA() - PI;
	if( aoa < -PI )
		aoa+=2.0*PI;
	
	slip = GetSlipAngle() - PI;
	if( slip < -PI )
		slip += 2.0*PI;

	if( aoa > 30*RAD )
		aoa = 30*RAD;
	else if( aoa < -30*RAD )
		aoa = -30*RAD;

	if( slip > 30*RAD )
		slip = 30*RAD;
	else if( slip < -30*RAD )
		slip = -30*RAD;
	
	aoa /= 30*RAD;
	slip /= 30*RAD;  // -1 to 1

	aoa /= 2.0;
	slip /= 2.0;	// -0.5 to 0.5

	aoa += 0.5;		// 0 to 1
	slip += 0.5;

	slip = 1.0-slip;   // correct for opposite sign
/*
	if( animcomp1 != NULL ) {
		SetAnimation( anim1, aoa );
	}
	if( animcomp2 != NULL ) {
		SetAnimation( anim2, slip );
	}
*/
}

void TMA::UpdateMesh ()
{
	// update animation states
	if (mesh_tma != MESH_UNDEFINED) {
		Animations(mesh_tma);
	}

}

void TMA::Animations(UINT mshid) {

//	original code - untouched
	int i;

	for( i = 0 ; i < 20 ; i++ ) {
		DelAnimationComponent(anim3, animcomp3[i] );
	}

	MGROUP_ROTATE *mgroup;
	MGROUP_TRANSLATE *mgroupt;

 	/*** left panel */
	
	static UINT groups0[1] = { 16 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups0, 1,
			_V(-1.1,0,0),
			_V(0,0,1),
			90*RAD
		);
	animcomp3[0] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup);
	
	static UINT groups1[1] = { 17 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups1, 1,
			_V(-1.12,0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[1] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[0]);

	static UINT groups2[1] = { 20 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups2, 1,
			_V(-1.15,-0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[2] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[1]);

		static UINT groups3[2] = { 19, 18 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups3, 2,
			_V(-1.18,0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[3] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[2]);


	/*** right panel */
	
	static UINT groups4[1] = { 21 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups4, 1,
			_V(1.1,0,0),
			_V(0,0,1),
			-90*RAD
		);
	animcomp3[4] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup);
	
	static UINT groups5[1] = { 22 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups5, 1,
			_V(1.12,0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[5] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[4]);

	static UINT groups6[1] = { 25 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups6, 1,
			_V(1.15,-0.63,0),
			_V(0,0,1),
			-178*RAD
		);
	animcomp3[6] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[5]);

	static UINT groups7[2] = { 23,24 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups7, 2,
			_V(1.18,0.63,0),
			_V(0,0,1),
			178*RAD
		);
	animcomp3[7] = AddAnimationComponent( anim3, 0.1, 0.9, mgroup, animcomp3[6]);


	/* antennae */


	static UINT groups8[1] = { 13 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups8, 1,
			_V(-0.722,0.891,-2.6),
			_V(0.239796,0.97082,0),
			178*RAD
		);
	animcomp3[8] = AddAnimationComponent( anim3, 0.05, 0.3, mgroup);

	static UINT groups9[1] = { 15 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups9, 1,
			_V(0,0.94,4.11),
			_V(1,0,0),
			-140*RAD
		);
	animcomp3[9] = AddAnimationComponent( anim3, 0.31, 0.85, mgroup);

	static UINT groups10[1] = { 15 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups10, 1,
			_V(0,0.94,4.11),
			_V(1,0,0),
			-10*RAD
		);
	animcomp3[10] = AddAnimationComponent( anim3, 0.27, 0.3, mgroup);

	
		
	static UINT groups11[1] = { 14 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups11, 1,
			_V(-0.94,0,4.11),
			_V(0,1,0),
			-130*RAD
		);
	animcomp3[11] = AddAnimationComponent( anim3, 0.3, 0.9, mgroup);

	static UINT groups12[1] = { 14 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups12, 1,
			_V(-0.94,0,4.11),
			_V(0,1,0),
			-20*RAD
		);
	animcomp3[12] = AddAnimationComponent( anim3, 0.3, 0.35, mgroup);

	static UINT groups13[2] = { 3,5 };
	mgroupt = new MGROUP_TRANSLATE (
			mshid,
			groups13, 2,
			_V(0,-0.4,0.07)
		);
	animcomp3[13] = AddAnimationComponent( anim3, 0.8, 1.0, mgroupt);


	/* thingie */

	static UINT groups14[1] = { 27 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups14, 1,
			_V(0.565,-0.35,4.36),
			_V(0.574,1,0),
			-50*RAD
		);
	animcomp3[14] = AddAnimationComponent( anim3, 0.02, 0.1, mgroup);

	static UINT groups15[1] = { 26 };
	mgroup = new MGROUP_ROTATE (
			mshid,
			groups15, 1,
			_V(0.794,-0.484,4.093),
			_V(0.574,1,0),
			50*RAD
		);
	animcomp3[15] = AddAnimationComponent( anim3, 0.02, 0.1, mgroup, animcomp3[14]);

}

void TMA::makeanimation( int i, UINT groups[], int n, VECTOR3 ref, VECTOR3 axis, double range , double start = 0, double end = 1, UINT mid = 0) {

//	original code - untouched
//	unknown function
	if( animlevel > 0.5 ) {
		animlevel = 1.0;
	}
	else {
		animlevel = 0.0;
	}

	
	rot[i] = new MGROUP_ROTATE (
		mid, // mesh index
		groups, n, // group list and # groups
		ref, // rotation reference point
		axis, // rotation axis
		(float) range // angular rotation range
	);
	animcomp3[i] = AddAnimationComponent (anim3, start, end, rot[i]);

}

// ==============================================================
// Overloaded callback functions
// ==============================================================

void TMA::clbkSetClassCaps (FILEHANDLE cfg)
{

	if (!oapiReadItem_bool (cfg, "RenderCockpit", render_cockpit))
		render_cockpit = false;

}

void TMA::clbkSetStateEx (const void *status)
{
	// default parameter initialisation
	DefSetStateEx (status);

	// reset vessel-specific parameters to defaults
	SetMainStage();
	refresh = 1;

}

void TMA::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{

    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		if( !strnicmp( line, "CONFIGURATION", 13 ) ) {
            sscanf (line+13, "%d", &status);
		}
		else if( !strnicmp( line, "PANELS", 6 ) ) {
			sscanf (line+6, "%lf", &animlevel);
			animlevel = atof( line+6 );
		}
		else if( !strnicmp( line, "SHIELD", 6 ) ) {
            sscanf (line+6, "%d", &shieldstatus);
		}
		else
		{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
//		ParsePayloadString(line);

			ParseScenarioLineEx (line, vs);
		}
	}

		if( animlevel > 0.5 )
			animactive = 1;
		else
			animactive = 2;

	/* always set the stage whenever something may have changed */
	switch (status) {
	case 0:		
	case 1:		// Soyuz TMA in orbit
		SetMainStage();
		break;
	case 2:		// Soyuz Descent Module
		SetEntryStage();
		break;
	case 3:		// Drogue deployment
		SetDrogueStage();
		break;
	case 4:		// Main chute deployment
		SetParaStage();
		break;
	case 5:		//Retro firing at approach to ground
		SetRetroStage();
		break;
	case 6:		// Touch down and sit.
		SetGroundedStage();
	}
	refresh = 1;

}

void TMA::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// custom parameters
	oapiWriteScenario_int (scn, "CONFIGURATION", status);

	oapiWriteScenario_int (scn, "SHIELD", shieldstatus);

	sprintf (cbuf, "%0.4f", animlevel);
	oapiWriteScenario_string (scn, "PANELS", cbuf);

//	SaveDefaultStateEx(scn);

}

void TMA::clbkFocusChanged (bool getfocus, OBJHANDLE newv, OBJHANDLE oldv)
{
	if (getfocus) {
		oapiDisableMFDMode (MFD_LANDING);
		// no VTOL MFD mode for Atlantis
	}
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}

void TMA::clbkPostStep (double simt, double simdt, double mjd)
{


	switch (status) {
	case 0:		// Soyuz TMA in orbit
			SetMainStage();
			break;
	case 1:		// Soyuz Descent Module
		if( animactive ) {
			if( animactive == 1 ) {
				animlevel += oapiGetSimStep() / 10.0;
				if( animlevel > 1.0 ) {
					animlevel = 1.0;
					animactive = 0;
				}
			}
			else  {
				animlevel -= oapiGetSimStep() / 10.0;
				if( animlevel < 0.0 ) {
					animlevel = 0.0;
					animactive = 0;
				}
			}
			SetAnimation( anim3, animlevel );
		}
		if( bManualSeparate) {
			BlowOrbiter();
			SetEntryStage();
			bManualSeparate = false;
		}
			break;
	case 2:		// Drogue deployment
		if( GetAltitude() < 12000 && GetAirspeed() < 300 ) {
		SetDrogueStage();
		}
			break;
	case 3:		// Main chute deployment
		CalcParaAngles();
//		if( oapiGetTimeAcceleration() > 10.0 ) {
//			oapiSetTimeAcceleration(10.0);
//		}
		if ( GetAltitude() < 2003.0 && shieldstatus == 0) {
			BlowHeatShield();
		}
		if ( GetAltitude() < 2000.0 && GetAirspeed() < 80.0 ) {
			oapiDeleteMesh (hdrogue);
			SetParaStage();
		}
			break;
	case 4:		//Retro firing at approach to ground
		CalcParaAngles();
//		if( oapiGetTimeAcceleration() > 10.0 ) {
//			oapiSetTimeAcceleration(10.0);
//		}
		if( GetAltitude() < 2.0) {
			oapiDeleteMesh (hpara);
			SetRetroStage();
			SetThrusterLevel(th_srm,1.0);
		}
			break;
	case 5:		// Touch down and sit.
		SetThrusterLevel(th_srm,1.0);
		if( GroundContact()) {
			SetThrusterLevel(th_srm,0.0);
			SetGroundedStage();
		}
	}

//	if( refresh > 0 ) {
//		refresh++;
//		if( refresh == 2 )
//			SetAnimation( anim3, 0.0 );
//		if( refresh == 3 )
//			if( status == 0 )
//				SetMainStage();
//			else if( status == 1 && bManualSeparate) {
//				BlowOrbiter();
//				SetEntryStage();
//				bManuelSeparate = false;
//			}
//			else if( status == 2 ) {
//				if( GetAltitude() < 12000 && GetAirspeed() < 300 ) {
//				SetDrogueStage();
//				}
//			}
//			else if( status == 3 ) {
//				CalcParaAngles();
//				if( oapiGetTimeAcceleration() > 10.0 ) {
//					oapiSetTimeAcceleration(10.0);
//				}
//				if ( GetAltitude() < 2000.0 && GetAirspeed() < 80.0 ) {
//					SetParaStage();
//				}
//			}
//			else if( status == 4 ) {
//				CalcParaAngles();
//				if( oapiGetTimeAcceleration() > 10.0 ) {
//					oapiSetTimeAcceleration(10.0);
//				}
//				if( GetAltitude() < 2.0) {
//					oapiDeleteMesh (hpara);
//					SetRetroStage();
//				}
//			}
//			else if( status == 5 && GroundContact()) {
//				SetGroundedStage();
//			}
//		if( refresh == 4 ) {
//			SetAnimation( anim3, animlevel );
//			refresh = 0;
//		}

//	}



//	if( status == 2 || status == 3 ) {
//		if( oapiGetTimeAcceleration() > 10.0 ) {
//			oapiSetTimeAcceleration(10.0);
//		}
//		CalcParaAngles();
//		if( status == 3 && GetAltitude() < 2.0 && th_srm != NULL ) {
//			oapiDeleteMesh (hpara);
//			//chute = JettisonComponent("SoyuzChute", "soychute", (VZERO-comshift)+_V(0,0,72) , _V(0,0,0), _V(0,0,0));
//			status = 4;
//			refresh = 1;
//		}
//		else if( status == 2 && (  ( GetAltitude() < 2000.0 && GetAirspeed() < 80.0 ) || GetAltitude() < 600.0 ) ) {
//			status = 3;
//			refresh = 1;
//		}
//	}
//	else if( status == 1 ) {
//		if( GetAltitude() < 12000 && GetAirspeed() < 300 ) {
//			status = 2;
//			refresh = 1;
//			return;
//		}
//	}

//	if( status == 4 && chute ) {
//		if( chute->GroundContact() ){
//			oapiDeleteVessel(chute->GetHandle());
//			chute = NULL;
//		}
//	}


}

bool TMA::clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event)
{

	return false;

}

void TMA::clbkVisualCreated (VISHANDLE _vis, int refcount)
{

	if (refcount > 1) return; // we don't support more than one visual per object
	vis = _vis;

	//UpdateMesh ();
}

void TMA::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{

	if (vis == _vis) vis = NULL;

}

void TMA::clbkAnimate (double simt)
{

//	UpdateMesh ();

}

bool TMA::clbkLoadVC (int id)
{

	switch (id) {
	case 0:
		oapiVCRegisterHUD (&huds);
		oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
		oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
		return true;
	default:
		return false;
	}

}

/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */
int TMA::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{

	if (!down) return 0; // only process keydown events


		switch (key) {
		case OAPI_KEY_J:  // "Jettison"
			if (!Playback()) {
				bManualSeparate = true;
			}
			return 1;
		case OAPI_KEY_K:  // "Solar Panels"
			if( animlevel > 0.5 )
				animactive = 2;
			else
				animactive = 1;
			return 1;
		}


//			VESSELSTATUS vs;
//	GetStatus(vs);
	
//	if (KEYMOD_SHIFT (kstate)) {
		// reserved for MFDs
//	}
//	else if (KEYMOD_CONTROL (kstate)) 
//	{
//		if (KEYDOWN (kstate, OAPI_KEY_J)  ) 
//		{ 
//			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				if( status == 0 ) {
//					bManualSeparate = true;
//				}
//				return 1;
//			}
//		}
//	}
//	else 
//	{ // unmodified keys
//		if (KEYDOWN (kstate, OAPI_KEY_K)) { // "Jettison Stage"
//			if(oapiAcceptDelayedKey (OAPI_KEY_K, 0.5)) {
//				if( animlevel > 0.5 )
//					animactive = 2;
//				else
//					animactive = 1;
//				return 1;
//			}
//		}
//		if (KEYDOWN (kstate, OAPI_KEY_J) && !KEYMOD_CONTROL(kstate) && !KEYMOD_SHIFT(kstate) ) 
//		{ 
//			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
//				DeployPayload();
//				refresh=1;
//				return 1;
//			}
//		}
//	}

	return 0;

}
// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Module initialisation
// --------------------------------------------------------------
DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new TMA (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (TMA*)vessel;
}
